from ftw.simplelayout.browser.ajax.utils import json_response
from ftw.simplelayout.images.cropping.behaviors import IImageCropping
from ftw.simplelayout.interfaces import ISimplelayoutBlock
from ftw.simplelayout.utils import get_block_html
from plone.app.uuid.utils import uuidToObject
from plone.dexterity.browser.edit import DefaultEditForm
from plone.dexterity.events import EditCancelledEvent
from plone.dexterity.events import EditFinishedEvent
from plone.dexterity.interfaces import IDexterityEditForm
from plone.dexterity.i18n import MessageFactory as DXMF
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from z3c.form import button
from zExceptions import BadRequest
from zope.event import notify
from zope.interface import classImplements
import json


class BlockEditRedirector(BrowserView):

    def __call__(self):
        payload = self.request.get('data', None)
        if not payload:
            raise BadRequest('No data given')

        data = json.loads(payload)
        block = uuidToObject(data['block'])

        return self.request.RESPONSE.redirect('{0}/edit.json'.format(
            block.absolute_url()))


class EditForm(DefaultEditForm):
    template = ViewPageTemplateFile('templates/edit_block_form.pt')

    _finished_edit = False

    @button.buttonAndHandler(DXMF(u'Save'), name='save')
    def handleApply(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return

        if IImageCropping.providedBy(self.context) and \
                self.context.image is not data.get('image'):
            data['IImageCropping.cropped_config'] = None
            data['IImageCropping.cropped_image'] = None

        self.applyChanges(data)
        notify(EditFinishedEvent(self.context))

        self._finished_edit = True

    @button.buttonAndHandler(DXMF(u'Cancel'), name='cancel')
    def handleCancel(self, action):
        notify(EditCancelledEvent(self.context))

    def get_block_content(self):
        return get_block_html(self.context)

    def render(self):

        response = {'content': self.template(),
                    'proceed': False}

        if self._finished_edit:
            response['proceed'] = True
            response['content'] = self.get_block_content()

        self.request.response.setHeader("Cache-Control",
                                        "no-cache, no-store, must-revalidate")
        self.request.response.setHeader("Expires", "Sat, 1 Jan 2000 00:00:00 GMT")

        return json_response(self.request, response)


classImplements(EditForm, IDexterityEditForm)


class InnerEditRedirector(BrowserView):

    def __call__(self):
        payload = self.request.get('data', None)
        if not payload:
            raise BadRequest('No data given')

        data = json.loads(payload)
        block = uuidToObject(data['uid'])

        return self.request.RESPONSE.redirect('{0}/inner_edit.json'.format(
            block.absolute_url()))


class InnerEditForm(EditForm):
    """Edit form for items in a folderish block"""

    def get_block_content(self):
        block = self.context.aq_parent
        if ISimplelayoutBlock.providedBy(block):
            return get_block_html(block)
        else:
            raise ValueError("The parent needs to be a simplelayout block")


classImplements(InnerEditForm, IDexterityEditForm)



from DateTime import DateTime
from ftw.builder import Builder
from ftw.builder import create
from ftw.simplelayout.images.limits.limits import ImageLimits
from ftw.simplelayout.interfaces import ISimplelayoutDefaultSettings
from ftw.simplelayout.testing import FTW_SIMPLELAYOUT_CONTENT_TESTING
from ftw.simplelayout.testing import SimplelayoutTestCase
from ftw.testbrowser import browsing
from ftw.testbrowser.tests.helpers import asset
from plone import api
from urlparse import parse_qs
from urlparse import urlparse
import transaction


class TestCropping(SimplelayoutTestCase):

    layer = FTW_SIMPLELAYOUT_CONTENT_TESTING

    def image_limits(self, config, context=None):
        limits = ImageLimits(context)
        limits.limit_configuration = config

        return limits

    @browsing
    def test_cropping_view_returns_json(self, browser):
        page = create(Builder('sl content page'))
        block = create(Builder('sl textblock').within(page).with_dummy_image())

        browser.login().visit(block, view='image_cropping.json')

        self.assertIsInstance(browser.json, dict)

    @browsing
    def test_default_buttons(self, browser):
        page = create(Builder('sl content page'))
        block = create(Builder('sl textblock').within(page).with_dummy_image())

        browser.login().visit(block, view='image_cropping.json')
        browser.open_html(browser.json.get('content'))

        self.assertTrue(browser.css('.btnDragModeMove'))
        self.assertTrue(browser.css('.btnDragModeCrop'))

        self.assertTrue(browser.css('.btnZoomIn'))
        self.assertTrue(browser.css('.btnZoomOut'))

        self.assertEqual(
            ['4:3', '16:9'],
            [el.text for el in browser.css('.btnAspectRatioButton')]
            )

        self.assertTrue(browser.css('.btnClear'))

    @browsing
    def test_ratio_buttons_are_generated_from_registry(self, browser):
        page = create(Builder('sl content page'))
        block = create(Builder('sl textblock').within(page).with_dummy_image())

        self._set_settings({
            block.portal_type: [u'free => 0']
        })

        browser.login().visit(block, view='image_cropping.json')
        browser.open_html(browser.json.get('content'))

        self.assertEqual(
            ['free'],
            [el.text for el in browser.css('.btnAspectRatioButton')]
            )

        self.assertEqual(
            ['0'],
            [el.get('data-value') for el in browser.css('.btnAspectRatioButton')]
            )

    @browsing
    def test_remove_cropped_image_if_main_image_has_changed(self, browser):
        page = create(Builder('sl content page'))
        block = create(Builder('sl textblock').within(page)
                       .with_cropped_image())

        self.assertIsNotNone(block.cropped_image)

        browser.login().visit(block, view='edit.json')
        browser.parse(browser.json['content'])

        with asset('mario.gif') as mario:
            browser.fill({'Image': (mario.read(), 'mario.gif')}).save()

        self.assertIsNone(block.cropped_image)

    def _set_settings(self, setting):
        api.portal.set_registry_record(
            name='image_cropping_aspect_ratios',
            value=setting,
            interface=ISimplelayoutDefaultSettings)

        transaction.commit()

    @browsing
    def test_append_last_modified_to_image_src_to_fix_caching_issues(self, browser):
        page = create(Builder('sl content page'))
        block = create(Builder('sl textblock').within(page).with_dummy_image())

        last_modified = DateTime(2014, 5, 7, 12, 30)
        block.setModificationDate(last_modified)

        transaction.commit()

        browser.login().visit(block, view='image_cropping.json')
        browser.open_html(browser.json.get('content'))
        image_src = browser.css('.croppingImage').first.attrib.get('src')
        query = parse_qs(urlparse(image_src).query)

        self.assertEqual(['last_modified'], query.keys())
        self.assertEqual([last_modified.strftime('%s')], query.get('last_modified'))



from ftw.upgrade import UpgradeStep


class ConfigurePageToContainAnchors(UpgradeStep):
    """Configure page to contain anchors.
    """

    def __call__(self):
        self.install_upgrade_profile()



from ftw.builder import Builder
from ftw.builder import create
from ftw.simplelayout.testing import FTW_SIMPLELAYOUT_CONTENT_TESTING
from ftw.simplelayout.utils import IS_PLONE_5
from ftw.testbrowser import browsing
from ftw.testbrowser.pages import factoriesmenu
from ftw.testbrowser.pages import statusmessages
from lxml import etree
from plone import api
from plone.app.textfield.value import RichTextValue
from plone.uuid.interfaces import IUUID
from unittest import TestCase
from unittest import skipUnless
from z3c.relationfield import RelationValue
from zope.component import getUtility
from zope.intid.interfaces import IIntIds
import difflib
import json
import transaction


class TestAliasBlockRendering(TestCase):

    layer = FTW_SIMPLELAYOUT_CONTENT_TESTING

    def setUp(self):
        self.page1 = create(Builder('sl content page'))
        self.page2 = create(Builder('sl content page'))
        self.textblock = create(Builder('sl textblock')
                                .titled(u'\xc4s Bl\xf6ckli')
                                .within(self.page1))

        self.intids = getUtility(IIntIds)

    def assert_html(self, page_html, aliasblock_html):
        diff = difflib.unified_diff(
            page_html.strip().split('\n'),
            aliasblock_html.strip().split('\n'),
            fromfile='page',
            tofile='aliasblock'
        )
        output = list(diff)
        if len(output) != 0:
            self.fail('HTML differs:\n{}'.format('\n'.join(output)))

    @browsing
    def test_create_aliasblock(self, browser):
        alias = create(Builder('sl aliasblock')
                       .having(alias=RelationValue(
                           self.intids.getId(self.textblock)))
                       .within(self.page2))

        browser.visit(self.page2)
        block_elements = browser.visit(alias).css('.sl-alias-block')

        self.assertEqual(len(block_elements), 1)

    @browsing
    def test_add_aliasblock_using_factoriesmenu(self, browser):
        browser.login().visit(self.page2)
        factoriesmenu.add('AliasBlock')

        form = browser.find_form_by_field('Alias Content')
        form.find_widget('Alias Content').fill(self.textblock)
        browser.find_button_by_label('Save').click()

        self.assertTrue(browser.css('.sl-alias-block'))

    @browsing
    def test_render_textblock_in_aliasblock(self, browser):
        create(Builder('sl aliasblock')
               .having(alias=RelationValue(
                       self.intids.getId(self.textblock)))
               .within(self.page2))

        browser.login().visit(self.page2)
        visit_link = browser.css('.sl-alias-block-visit-block').first.text
        textblock_title = browser.css('.sl-alias-block h2').first.text

        self.assertEqual(u'\U0001f517 Visit embedded block', visit_link)
        self.assertEqual(u'\xc4s Bl\xf6ckli', textblock_title)

    @browsing
    def test_has_message_if_referenced_block_was_deleted(self, browser):
        create(Builder('sl aliasblock')
               .having(alias=RelationValue(
                       self.intids.getId(self.textblock)))
               .within(self.page2))

        api.content.delete(obj=self.textblock, check_linkintegrity=False)
        transaction.commit()
        browser.visit(self.page2)
        block_content = browser.css('.sl-alias-block').first.text

        self.assertEqual(u'The content is no longer accessible to you', block_content)

    @browsing
    @skipUnless(IS_PLONE_5, 'Link integrity check for DX only works with Plone 5')
    def test_link_integrity_message_if_target_will_be_deleted(self, browser):
        create(Builder('sl aliasblock')
               .having(alias=RelationValue(
                       self.intids.getId(self.textblock)))
               .within(self.page2))

        payload = {'data': json.dumps({'block': IUUID(self.textblock)})}
        browser.login().visit(self.page1,
                              view='@@sl-ajax-delete-blocks-view',
                              data=payload)

        browser.parse(browser.json['content'])
        self.assertEquals(
            'These internal links will be broken',
            browser.css('#content-core > div p').first.text
        )

        self.assertEquals(
            'These internal links will be broken',
            browser.css('#content-core > div p').first.text
        )

        self.assertEquals(
            u'AliasBlock: "{}"'.format(self.textblock.Title().decode('utf-8')),
            browser.css('#content-core > div ul li a').first.text
        )

    @browsing
    def test_custom_selectable_implementation(self, browser):
        folder = create(Builder('folder')
                        .titled(u'A folder'))

        browser.login().visit(self.page1)
        factoriesmenu.add('AliasBlock')

        form = browser.find_form_by_field('Alias Content')
        form.find_widget('Alias Content').fill(folder)

        browser.find_button_by_label('Save').click()
        self.assertEquals(['There were some errors.'],
                          statusmessages.error_messages())
        self.assertEquals('The selected content cannot be selected',
                          browser.css('#formfield-form-widgets-alias .error').first.text)

    @browsing
    def test_dont_allow_pages_with_alias_block(self, browser):
        browser.login().visit(self.page1)
        alias = create(Builder('sl aliasblock')
                       .having(alias=RelationValue(
                           self.intids.getId(self.textblock)))
                       .within(self.page2))

        browser.login().visit(self.page1)
        factoriesmenu.add('AliasBlock')

        form = browser.find_form_by_field('Alias Content')
        form.find_widget('Alias Content').fill(self.page2)

        browser.find_button_by_label('Save').click()
        self.assertEquals(['There were some errors.'],
                          statusmessages.error_messages())
        self.assertEquals('The selected ContentPage contains a Aliasblock or is the '
                          'page you are creating the block on and thus cannot be selected',
                          browser.css('#formfield-form-widgets-alias .error').first.text)

    @browsing
    def test_dont_allow_creating_block_on_current_page(self, browser):
        browser.login().visit(self.page1)
        factoriesmenu.add('AliasBlock')

        form = browser.find_form_by_field('Alias Content')
        form.find_widget('Alias Content').fill(self.page1)

        browser.find_button_by_label('Save').click()
        self.assertEquals(['There were some errors.'],
                          statusmessages.error_messages())
        self.assertEquals('The selected ContentPage contains a Aliasblock or is the '
                          'page you are creating the block on and thus cannot be selected',
                          browser.css('#formfield-form-widgets-alias .error').first.text)

    @browsing
    def test_aliasblock_renders_contentpage(self, browser):
        create(Builder('sl textblock')
               .titled(u'\xc4s Bl\xf6ckli')
               .having(text=RichTextValue(u'<p>Some html</p>'))
               .within(self.page1))
        listing = create(Builder('sl listingblock')
                         .titled(u'Downloads')
                         .within(self.page1))
        create(Builder('file')
               .titled(u'A file')
               .with_dummy_content()
               .within(listing))

        create(Builder('sl videoblock')
               .having(video_url='https://youtu.be/W42x6-Wf3Cs')
               .within(self.page1))

        create(Builder('sl aliasblock')
               .having(alias=RelationValue(
                       self.intids.getId(self.page1)))
               .within(self.page2))

        browser.login().visit(self.page1)
        html_page1_layout1 = etree.tostring(
            browser.css('#content-core #default .sl-layout').first.node,
            pretty_print=True
        )
        browser.visit(self.page2)
        html_aliasblock_layout1 = etree.tostring(
            browser.css('.sl-alias-block .sl-layout').first.node,
            pretty_print=True
        )
        self.assert_html(html_page1_layout1, html_aliasblock_layout1)

    @browsing
    def test_respect_view_permission(self, browser):
        alias = create(Builder('sl aliasblock')
                       .having(alias=RelationValue(
                           self.intids.getId(self.textblock)))
                       .within(self.page2))

        self.page1.manage_permission('View', roles=[])
        self.page1.reindexObjectSecurity()
        transaction.commit()
        browser.login().visit(self.page2)
        self.assertEquals('The content is no longer accessible to you',
                          browser.css('.sl-alias-block p').first.text)

    @browsing
    def test_google_api_key_is_on_add_and_edit_form(self, browser):
        browser.login().visit(self.page2, view='++add_block++ftw.simplelayout.AliasBlock')
        browser.parse(browser.json['content'])
        self.assertTrue(browser.css('[data-googlejs]'), 'Googlejs should be there.')

        aliasblock = create(Builder('sl aliasblock')
                            .having(alias=RelationValue(
                                self.intids.getId(self.textblock)))
                            .within(self.page2))

        browser.visit(aliasblock, view='edit.json')
        browser.parse(browser.json['content'])
        self.assertTrue(browser.css('[data-googlejs]'), 'Googlejs should be there.')



from Products.CMFPlone.interfaces import IPloneSiteRoot
from ftw.simplelayout.interfaces import IBlockConfiguration
from ftw.simplelayout.interfaces import IPageConfiguration
from ftw.simplelayout.interfaces import ISimplelayout
from ftw.simplelayout.interfaces import ISimplelayoutBlock
from ftw.simplelayout.interfaces import ISimplelayoutLayer
from persistent.list import PersistentList
from persistent.mapping import PersistentMapping
from plone import api
from plone.dexterity.interfaces import IDexterityContainer
from plone.restapi.interfaces import ISerializeToJson
from plone.restapi.serializer.dxcontent import SerializeFolderToJson
from plone.restapi.serializer.dxcontent import SerializeToJson
from plone.restapi.serializer.site import SerializeSiteRootToJson
from zope.component import adapter
from zope.component import getMultiAdapter
from zope.interface import Interface
from zope.interface import implementer
import json


class PersistenceDecoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, PersistentMapping):
            return dict(obj)
        elif isinstance(obj, PersistentList):
            return list(obj)
        return json.JSONEncoder.default(self, obj)


def _sl_blocks_query(context):
    return {
        'path': {'depth': 1, 'query': '/'.join(context.getPhysicalPath())},
        'object_provides': ISimplelayoutBlock.__identifier__,
    }


def enrich_with_simplelayout(context, result):

    result['simplelayout'] = json.loads(json.dumps(
        IPageConfiguration(context).load(),
        cls=PersistenceDecoder)
    )

    catalog = api.portal.get_tool('portal_catalog')
    brains = catalog(_sl_blocks_query(context))

    blocks = getMultiAdapter(
        (brains, context.REQUEST), ISerializeToJson
    )(fullobjects=True)["items"]
    result['slblocks'] = {block['UID']: block for block in blocks}


@implementer(ISerializeToJson)
@adapter(ISimplelayout, Interface)
class SimplelayoutSerializer(SerializeFolderToJson):
    def __call__(self, version=None, include_items=True):
        result = super(SimplelayoutSerializer, self).__call__(version=version, include_items=include_items)
        enrich_with_simplelayout(self.context, result)
        return result


@implementer(ISerializeToJson)
@adapter(IPloneSiteRoot, ISimplelayoutLayer)
class SimplelayoutSerializeSiteRootToJson(SerializeSiteRootToJson):
    def __call__(self, version=None):
        result = super(SimplelayoutSerializeSiteRootToJson, self).__call__(version=version)
        enrich_with_simplelayout(self.context, result)
        return result


@implementer(ISerializeToJson)
@adapter(ISimplelayoutBlock, Interface)
class SimplelayoutBlockSerializeToJson(SerializeToJson):
    def __call__(self, version=None, include_items=True):
        result = super(SimplelayoutBlockSerializeToJson, self).__call__(version=version, include_items=include_items)

        result['block-configuration'] = json.loads(json.dumps(
            IBlockConfiguration(self.context).load(),
            cls=PersistenceDecoder)
        )

        if IDexterityContainer.providedBy(self.context):
            result['items'] = SerializeFolderToJson(self.context, self.request)()['items']

        return result



from ftw.upgrade import UpgradeStep


class AddNewGalleryBlockImageScaling(UpgradeStep):
    """Add new gallery block image scaling.
    """

    def __call__(self):
        self.install_upgrade_profile()



from ftw.testing import IS_PLONE_5
from ftw.testing.genericsetup import GenericSetupUninstallMixin
from ftw.testing.genericsetup import apply_generic_setup_layer
from unittest import TestCase
from unittest import skipIf
from unittest import skipUnless


@apply_generic_setup_layer
@skipIf(IS_PLONE_5, 'Test the uninstall profile for Plone 4')
class TestGenericSetupUninstallMapBlock(TestCase, GenericSetupUninstallMixin):
    package = 'ftw.simplelayout.mapblock'


@apply_generic_setup_layer
# @skipUnless(IS_PLONE_5, 'Test the uninstall profile for Plone 5')
@skipUnless(False, 'Enable this test a soon as the mapblick uninstalls cleanly')
class TestGenericSetupUninstallMapBlockPlone5(TestCase, GenericSetupUninstallMixin):
    package = 'ftw.simplelayout.mapblock'

    # Plone 5 does not support "propertiestool.xml" anymore.
    skip_files = ('propertiestool.xml',)


@apply_generic_setup_layer
@skipIf(IS_PLONE_5, 'Test the uninstall profile for Plone 4')
class TestGenericSetupUninstallContentTypes(TestCase, GenericSetupUninstallMixin):
    package = 'ftw.simplelayout.contenttypes'


@apply_generic_setup_layer
@skipUnless(IS_PLONE_5, 'Test the uninstall profile for Plone 5')
class TestGenericSetupUninstallContentTypesPlone5(TestCase, GenericSetupUninstallMixin):
    package = 'ftw.simplelayout.contenttypes'

    # Plone 5 does not support "propertiestool.xml" anymore.
    skip_files = ('propertiestool.xml',)


@apply_generic_setup_layer
class TestGenericSetupUninstallAliasBlock(TestCase, GenericSetupUninstallMixin):
    package = 'ftw.simplelayout.aliasblock'

    if IS_PLONE_5:
        # Plone 5 does not support "propertiestool.xml" anymore.
        skip_files = ('propertiestool.xml',)



"""Migration from ftw.contentpage to ftw.simplelayout.
The migration is not registered / executed automatically.
It must be executed manually from an upgrade step in an upgrade step of the
integration project.

Be aware that pages must be migrated before migrating blocks!

The types of ftw.contentpage are split up into multiple packages:

# ftw.simplelayout:
ContentPage => ftw.simplelayout.ContentPage
ListingBlock => ftw.simplelayout.FileListingBlock
                ftw.simplelayout.GalleryBlock
                ftw.sliderblock.SliderBlock
TextBlock => ftw.simplelayout.TextBlock

# ftw.addressblock
AddressBlock => ftw.addressblock.AddressBlock

# ftw.news
News => ftw.news.News
NewsFolder => ftw.news.NewsFolder

# ftw.events
EventPage => ftw.events.EventPage
EventFolder => ftw.events.EventFolder
"""

from ftw.upgrade import UpgradeStep
from ftw.upgrade.migration import InplaceMigrator

# Some sources will probably be removed after running the migration.
# But if the imports are kept and the sources are removed, the imports
# may raise ImportErrors. We only want to have those ImportErrors when
# we actually need to run the migration.
try:

    from archetypes.schemaextender.extender import CACHE_KEY
    from ftw.file.content.file import File
    from ftw.simplelayout.interfaces import IBlockConfiguration
    from ftw.simplelayout.interfaces import IPageConfiguration
    from ftw.simplelayout.interfaces import ISimplelayoutBlock as INewSLBlock
    from ftw.upgrade.migration import DUBLIN_CORE_IGNORES
    from functools import partial
    from operator import attrgetter
    from plone.app.blob.interfaces import IBlobWrapper
    from plone.app.event.dx.behaviors import IEventBasic
    from plone.dexterity.utils import createContentInContainer
    from plone.dexterity.utils import iterSchemata
    from plone.uuid.interfaces import IUUID
    from simplelayout.base.interfaces import IBlockConfig
    from simplelayout.base.interfaces import ISimpleLayoutBlock
    from simplelayout.base.interfaces import ISimplelayoutTwoColumnOneOnTopView
    from simplelayout.base.interfaces import ISimplelayoutTwoColumnView
    from simplelayout.base.interfaces import ISimplelayoutView
    from simplelayout.base.interfaces import ISlotA
    from simplelayout.base.interfaces import ISlotB
    from simplelayout.base.interfaces import ISlotC
    from simplelayout.base.interfaces import ISlotD
    from simplelayout.portlet.dropzone.interfaces import IPortletColumn
    from simplelayout.portlet.dropzone.interfaces import ISlotBlock
    from zope.component import implementedBy
    from zope.component import providedBy
    from zope.interface import noLongerProvides
    from zope.interface import alsoProvides
    from zope.schema import getFieldsInOrder

except ImportError, IMPORT_ERROR:
    pass
else:
    IMPORT_ERROR = None



class ExampleUpgradeStep(UpgradeStep):
    """This is just an example how an upgrade could be done in an upgrade
    step in the integration package.
    """

    def __call__(self):
        page_migrator = ContentPageMigrator()
        map(page_migrator.migrate_object,
            self.objects({'query': 'ContentPage'}), 'Migrate pages')

        textblock_migrator = TextBlockMigrator()
        map(textblock_migrator.migrate_object,
            self.objects({'query': 'TextBlock'}), 'Migrate text blocks')


LEAD_IMAGE_FIELDS = (
    # Lead image is migrated into text block.
    'image',
    'imageAltText',
    'imageCaption',
    'imageClickable',
)


SL_BLOCK_DEFAULT_IGNORED_FIELDS = (
    'effectiveDate',
    'excludeFromNav',
    'expirationDate',
    'subject',
)


def move_sl_block_into_slot(old_page, new_page, block, slot_name):
    page_configuration = IPageConfiguration(new_page)
    page_state = page_configuration.load()

    # initiate layout if it is in its initial state
    if slot_name == 'default' and page_state == page_configuration._default_page_config():

        # setup different layouts
        if ISimplelayoutTwoColumnView.providedBy(old_page):
            # two columns
            page_state[slot_name] = [
                {'cols': [ {'blocks': []}, {'blocks': []} ]}
            ]

        elif ISimplelayoutTwoColumnOneOnTopView.providedBy(old_page):
            # two columns and a top row
            page_state[slot_name] = [
                {'cols': [{'blocks': []}]},
                {'cols': [ {'blocks': []}, {'blocks': []} ]}
            ]

    if slot_name not in page_state:
        # normal single column layout
        page_state[slot_name] = [{'cols': [{'blocks': []}]}]

    if slot_name == 'default':
        slot = page_state['default']

        # two column layout
        if ISimplelayoutTwoColumnView.providedBy(old_page):
            if ISlotA.providedBy(block):  # left column
                slot[0]['cols'][0]['blocks'].append({'uid': IUUID(block)})
            elif ISlotB.providedBy(block):  # right column
                slot[0]['cols'][1]['blocks'].append({'uid': IUUID(block)})
            else:
                raise ValueError('Block has unused slot in layout.')

        # two columns and a top row layout
        elif ISimplelayoutTwoColumnOneOnTopView.providedBy(old_page):
            if ISlotA.providedBy(block):  # top row
                slot[0]['cols'][0]['blocks'].append({'uid': IUUID(block)})
            elif ISlotB.providedBy(block):  # bottom row, left column
                slot[1]['cols'][0]['blocks'].append({'uid': IUUID(block)})
            elif ISlotC.providedBy(block):  # bottom row, right column
                slot[1]['cols'][1]['blocks'].append({'uid': IUUID(block)})
            else:
                raise ValueError('Block has unused slot in layout.')

        else:
            slot[0]['cols'][0]['blocks'].append({'uid': IUUID(block)})

    else:
        page_state[slot_name][0]['cols'][0]['blocks'].append({
            'uid': IUUID(block)})

    page_configuration.store(page_state)


def migrate_sl_image_layout(old_object, new_object):
    block_layout_mapping = {
        'small': {
            'scale': 'sl_textblock_small',
            'imagefloat': 'left'},
        'middle': {
            'scale': 'sl_textblock_middle',
            'imagefloat': 'left'},
        'full': {
            'scale': 'sl_textblock_large',
            'imagefloat': 'no-float'},
        'middle-right': {
            'scale': 'sl_textblock_middle',
            'imagefloat': 'right'},
        'small-right': {
            'scale': 'sl_textblock_small',
            'imagefloat': 'right'},
        'no-image': {
            'scale': 'sl_textblock_small',
            'imagefloat': 'left'},
    }

    old_config = IBlockConfig(old_object)
    image_layout = old_config.get_image_layout()
    if not image_layout or image_layout == 'dummy-dummy-dummy':
        return

    new_config = IBlockConfiguration(new_object)
    cfg = new_config.load()
    cfg.update(block_layout_mapping[image_layout])
    new_config.store(cfg)


def migrate_simplelayout_page_state(old_page, new_page):
    for block in new_page.listFolderContents():
        if INewSLBlock.providedBy(block):
            raise ValueError('Block must be migrated after pages.')

        if not ISimpleLayoutBlock.providedBy(block):
            continue

        if ISlotBlock.providedBy(block) or IPortletColumn.providedBy(block):
            move_sl_block_into_slot(old_page, new_page, block, 'portletright')

        elif ISlotD.providedBy(block):
            move_sl_block_into_slot(old_page, new_page, block, 'bottom')

        else:
            move_sl_block_into_slot(old_page, new_page, block, 'default')


def migrate_lead_image_into_textblock(old_page, new_page):
    image = old_page.getImage()
    if not image or not image.get_size():
        # no lead image
        return

    assert IBlobWrapper.providedBy(image), \
        'Unexpectedly, the teaser image "{!r}" is not an IBlobWrapper.'.format(
            image)

    teaser_block = createContentInContainer(
        container=new_page,
        portal_type='ftw.simplelayout.TextBlock',
        title='Teaser'
    )

    fields = dict(reduce(list.__add__,
                         map(getFieldsInOrder, iterSchemata(teaser_block))))

    fields['show_title'].set(teaser_block, False)
    fields['image_alt_text'].set(
        teaser_block, old_page.getImageAltText().decode('utf-8'))
    fields['image_caption'].set(
        teaser_block, old_page.getImageCaption().decode('utf-8'))
    fields['open_image_in_overlay'].set(
        teaser_block, old_page.getImageClickable())

    new_image = fields['image']._type(
        data='',
        contentType=image.content_type,
        filename=image.filename.decode('utf-8'))
    new_image._blob = image.getBlob()
    fields['image'].set(teaser_block, new_image)

    # Since the teaser was stored on the *page*, the image layout of the
    # teaser is stored in the block-config of the page!
    # Therefore we must migrate the view from the page to the new
    # teaser block.
    migrate_sl_image_layout(old_page, teaser_block)
    alsoProvides(teaser_block, ISlotA)
    move_sl_block_into_slot(old_page, new_page, teaser_block, 'default')


def migrate_image_to_file(obj):
    image = obj.getImage()

    # Use the functionality of the UpgradeStep
    class MigrateToFile(UpgradeStep):
        def __call__(self):
            # This is an AT => AT migration; we just replace the class.
            self.migrate_class(obj, File)

    MigrateToFile(obj.portal_setup)

    # clear archetypes.schemaextender cache
    archetypes_schemaextender_cache = getattr(obj.REQUEST, CACHE_KEY)
    if archetypes_schemaextender_cache:
        archetypes_schemaextender_cache.pop(IUUID(obj), None)

    ifaces_to_remove = (set(providedBy(obj)) -
                        set(implementedBy(obj.__class__)))
    map(partial(noLongerProvides, obj), ifaces_to_remove)

    obj.Schema()
    obj.setFile(image)
    obj.setDocumentDate(obj.created())
    obj.portal_type = 'File'


class ContentPageMigrator(InplaceMigrator):

    def __init__(self, ignore_fields=(), additional_steps=(), options=0,
                 field_mapping=None):
        if IMPORT_ERROR:
            raise IMPORT_ERROR

        super(ContentPageMigrator, self).__init__(
            new_portal_type='ftw.simplelayout.ContentPage',
            ignore_fields=(
                DUBLIN_CORE_IGNORES
                + LEAD_IMAGE_FIELDS
                + ('subject',)
                + ignore_fields),
            field_mapping=field_mapping,
            additional_steps=(
                (migrate_simplelayout_page_state,
                 migrate_lead_image_into_textblock)
                + additional_steps),
            options=options,
        )

    def query(self, path=None):
        query = {'portal_type': 'ContentPage'}
        if path:
            query['path'] = path
        return query


class NewsFolderMigrator(InplaceMigrator):

    def __init__(self, ignore_fields=(), additional_steps=(), options=0,
                 field_mapping=None):
        if IMPORT_ERROR:
            raise IMPORT_ERROR

        super(NewsFolderMigrator, self).__init__(
            new_portal_type='ftw.news.NewsFolder',
            ignore_fields=(
                DUBLIN_CORE_IGNORES
                + ignore_fields
                + ('effectiveDate',
                   'expirationDate',
                   'subject')),
            field_mapping=field_mapping,
            additional_steps=(
                (self.create_news_listing_block, )
                + additional_steps),
            options=options,
        )

    def query(self, path=None):
        query = {'portal_type': 'NewsFolder'}
        if path:
            query['path'] = path
        return query

    def create_news_listing_block(self, old_news_folder, new_news_folder):
        from ftw.news.contents.news_folder import create_news_listing_block
        create_news_listing_block(new_news_folder)


class NewsMigrator(InplaceMigrator):

    def __init__(self, ignore_fields=(), additional_steps=(), options=0,
                 field_mapping=None):
        if IMPORT_ERROR:
            raise IMPORT_ERROR

        super(NewsMigrator, self).__init__(
            new_portal_type='ftw.news.News',
            ignore_fields=(
                DUBLIN_CORE_IGNORES
                + LEAD_IMAGE_FIELDS
                + ignore_fields
                + (
                    'content_categories',  # no longer exists
                    'excludeFromNav',  # all news are excluded by default
                    'topics',  # no topics for news
                )),
            field_mapping=field_mapping,
            additional_steps=(
                (migrate_simplelayout_page_state,
                 migrate_lead_image_into_textblock)
                + additional_steps),
            options=options,
        )

    def query(self, path=None):
        query = {'portal_type': 'News'}
        if path:
            query['path'] = path
        return query


class EventFolderMigrator(InplaceMigrator):

    def __init__(self, ignore_fields=(), additional_steps=(), options=0,
                 field_mapping=None):
        if IMPORT_ERROR:
            raise IMPORT_ERROR

        super(EventFolderMigrator, self).__init__(
            new_portal_type='ftw.events.EventFolder',
            ignore_fields=(
                DUBLIN_CORE_IGNORES
                + ignore_fields
                + ('effectiveDate',
                   'expirationDate',
                   'subject')),
            field_mapping=field_mapping,
            additional_steps=(
                (self.create_event_listing_block, )
                + additional_steps),
            options=options,
        )

    def query(self, path=None):
        query = {'portal_type': 'EventFolder'}
        if path:
            query['path'] = path
        return query

    def create_event_listing_block(self, old_event_folder, new_event_folder):
        from ftw.events.contents.eventfolder import create_event_listing_block
        create_event_listing_block(new_event_folder)


class EventPageMigrator(InplaceMigrator):

    def __init__(self, ignore_fields=(), additional_steps=(), options=0,
                 field_mapping=None):
        if IMPORT_ERROR:
            raise IMPORT_ERROR

        field_mapping = field_mapping or {}
        field_mapping.update({
            'endDate': 'end',
            'startDate': 'start',
            'wholeDay': 'whole_day'})

        super(EventPageMigrator, self).__init__(
            new_portal_type='ftw.events.EventPage',
            ignore_fields=(
                DUBLIN_CORE_IGNORES
                + LEAD_IMAGE_FIELDS
                + (
                    'content_categories',  # no longer exists
                    'excludeFromNav',  # all events are excluded by default
                    'topics',  # no topics for events
                )
                + ignore_fields),
            field_mapping=field_mapping,
            additional_steps=(
                (self.ensure_timezone,
                 migrate_simplelayout_page_state,
                 migrate_lead_image_into_textblock)
                + additional_steps),
            options=options,
        )

    def ensure_timezone(self, old_object, new_object):
        # The "timezone" field was removed in newer p.a.event versions.
        if IEventBasic(new_object).timezone:
            return

        # We do not use the behavior on purpose here in order to
        # bypass the timezone setter.
        new_object.timezone = u'Europe/Zurich'

    def query(self, path=None):
        query = {'portal_type': 'EventPage'}
        if path:
            query['path'] = path
        return query

class ImageToSliderPaneMigrator(InplaceMigrator):

    def __init__(self, ignore_fields=(), additional_steps=(), options=0,
                 field_mapping=None):
        if IMPORT_ERROR:
            raise IMPORT_ERROR

        super(ImageToSliderPaneMigrator, self).__init__(
            new_portal_type='ftw.slider.Pane',
            ignore_fields=(
                DUBLIN_CORE_IGNORES
                + SL_BLOCK_DEFAULT_IGNORED_FIELDS
                + ignore_fields
                + (
                    'description',  # was invisible
                )),
            field_mapping=field_mapping,
            additional_steps=additional_steps,
            options=options,
        )

    def migrate_object(self, old_object):
        if old_object.portal_type not in ('Image',):
            raise ValueError(
                'Unsupported source portal_type {!r}'
                .format(old_object.portal_type))
        return super(ImageToSliderPaneMigrator, self).migrate_object(old_object)


class TextBlockMigrator(InplaceMigrator):

    def __init__(self, ignore_fields=(), additional_steps=(), options=0,
                 field_mapping=None):
        if IMPORT_ERROR:
            raise IMPORT_ERROR

        field_mapping = field_mapping or {}
        field_mapping.update({
            'showTitle': 'show_title',
            'imageAltText': 'image_alt_text',
            'imageCaption': 'image_caption',
            'imageClickable': 'open_image_in_overlay',
            'teaserExternalUrl': 'external_link',
            'teaserReference': 'internal_link'})

        super(TextBlockMigrator, self).__init__(
            new_portal_type='ftw.simplelayout.TextBlock',
            ignore_fields=(
                DUBLIN_CORE_IGNORES
                + SL_BLOCK_DEFAULT_IGNORED_FIELDS
                + ignore_fields
                + (
                    'description',  # was invisible
                    'teaserSelectLink',  # selection is now automatic => drop
                )),
            field_mapping=field_mapping,
            additional_steps=(migrate_sl_image_layout,) + additional_steps,
            options=options,
        )

    def query(self, path=None):
        query = {'portal_type': 'TextBlock'}
        if path:
            query['path'] = path
        return query


class AddressBlockMigrator(InplaceMigrator):

    def __init__(self, ignore_fields=(), additional_steps=(), options=0,
                 field_mapping=None):
        if IMPORT_ERROR:
            raise IMPORT_ERROR

        field_mapping = field_mapping or {}
        field_mapping.update({
            'addressTitle': 'address_title',
            'extraAddressLine': 'extra_address_line',
            'extraAddressLine': 'extra_address_line',
            'openingHours': 'opening_hours',
            'openingHours': 'opening_hours',
            'zip': 'zip_code'})

        super(AddressBlockMigrator, self).__init__(
            new_portal_type='ftw.addressblock.AddressBlock',
            ignore_fields=(
                DUBLIN_CORE_IGNORES
                + SL_BLOCK_DEFAULT_IGNORED_FIELDS
                + ignore_fields
                + (
                    'description',
                    'showOpeningHours',
                    'showTitle',
                )),
            field_mapping=field_mapping,
            additional_steps=additional_steps,
            options=options,
        )

    def query(self, path=None):
        query = {'portal_type': 'AddressBlock'}
        if path:
            query['path'] = path
        return query


class HTMLBlockMigrator(InplaceMigrator):

    def __init__(self, ignore_fields=(), additional_steps=(), options=0,
                 field_mapping=None):
        if IMPORT_ERROR:
            raise IMPORT_ERROR

        field_mapping = field_mapping or {}
        field_mapping.update({
            'showTitle': 'show_title',
            'text': 'content'})

        super(HTMLBlockMigrator, self).__init__(
            new_portal_type='ftw.htmlblock.HtmlBlock',
            ignore_fields=(
                DUBLIN_CORE_IGNORES
                + SL_BLOCK_DEFAULT_IGNORED_FIELDS
                + ignore_fields
                + (
                    'description',  # was invisible
                )),
            field_mapping=field_mapping,
            additional_steps=additional_steps,
            options=options,
        )

    def query(self, path=None):
        query = {'portal_type': 'HTMLBlock'}
        if path:
            query['path'] = path
        return query


class ListingBlockMigrator(object):

    def __init__(self, ignore_fields=(), additional_steps=(), options=0,
                 field_mapping=None):
        if IMPORT_ERROR:
            raise IMPORT_ERROR

        # Old listing blocks may be migrated to new listing blocks or may
        # become gallery blocks when suitable.
        # https://github.com/4teamwork/izug.organisation/issues/890

        ignore_fields = DUBLIN_CORE_IGNORES + SL_BLOCK_DEFAULT_IGNORED_FIELDS + (
            'description',  # hidden thus unused
        ) + ignore_fields

        field_mapping = field_mapping or {}
        field_mapping.update({
            'showTitle': 'show_title',
            'sortOn': 'sort_on',
            'sortOrder': 'sort_order',
            'tableColumns': 'columns'})

        self.filelisting_migrator = InplaceMigrator(
            new_portal_type='ftw.simplelayout.FileListingBlock',
            ignore_fields=ignore_fields + ('slick_settings',),
            field_mapping=field_mapping,
            additional_steps=additional_steps,
            options=options,
        )

        self.gallery_migrator = InplaceMigrator(
            new_portal_type='ftw.simplelayout.GalleryBlock',
            ignore_fields=ignore_fields + (
                'slick_settings',
                'sortOn',
                'sortOrder',
                'tableColumns'),
            field_mapping=field_mapping,
            additional_steps=additional_steps,
            options=options,
        )

        self.sliderblock_migrator = InplaceMigrator(
            new_portal_type='ftw.sliderblock.SliderBlock',
            ignore_fields=ignore_fields + (
                'sortOn',
                'sortOrder',
                'tableColumns'),
            field_mapping={
                'showTitle': 'show_title',
                'crop_image': 'False',
                'slick_settings': 'slick_config'},
            additional_steps=additional_steps,
            options=options,
        )

        self.image_to_slider_pane_migrator = ImageToSliderPaneMigrator(
            ignore_fields=ignore_fields,
            field_mapping=field_mapping,
            options=options)

    def query(self, path=None):
        query = {'portal_type': 'ListingBlock'}
        if path:
            query['path'] = path
        return query

    def migrate_object(self, old_object):
        view = IBlockConfig(old_object).get_viewname()
        if view not in (None, '', 'gallery', 'dummy', 'block_view', 'slider'):
            raise ValueError('ListingBlock: unexpected view {!r}'.format(view))

        children_types = set(map(attrgetter('portal_type'),
                                 old_object.objectValues()))

        if view == 'slider' and children_types == {'Image'}:
            map(self.image_to_slider_pane_migrator.migrate_object,
                old_object.objectValues()[:])
            self.sliderblock_migrator.migrate_object(old_object)

        elif not children_types or children_types == {'File'}:
            self.filelisting_migrator.migrate_object(old_object)

        elif children_types == {'Image'}:
            self.gallery_migrator.migrate_object(old_object)

        elif children_types == {'File', 'Image'}:
            # convert all "Image" children to "File"
            map(migrate_image_to_file,
                filter(lambda child: child.portal_type=='Image',
                       old_object.objectValues()))

            self.filelisting_migrator.migrate_object(old_object)

        else:
            raise ValueError('ListingBlock: unexpected children: {}'.format(
                children_types))



from Acquisition import aq_base
from Acquisition import aq_inner
from ftw.simplelayout.contenttypes.behaviors import IHiddenBlock
from ftw.simplelayout.interfaces import IPageConfiguration
from ftw.simplelayout.interfaces import ISimplelayoutContainerConfig
from ftw.simplelayout.interfaces import ISimplelayoutDefaultSettings
from ftw.simplelayout.utils import get_block_html
from ftw.simplelayout.utils import normalize_portal_type
from persistent.mapping import PersistentMapping
from plone import api
from plone.registry.interfaces import IRegistry
from Products.CMFCore.utils import getToolByName
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from ZODB.POSException import ConflictError
from zope.component import getUtility
from zope.component import queryMultiAdapter
from zope.contentprovider.interfaces import ITALESProviderExpression
from zope.interface import implements
from zope.tales import expressions
import json
import logging
import re


LOG = logging.getLogger('ftw.simplelayout')


class CleanedViewPageTemplateFile(ViewPageTemplateFile):
    """Remove spaces and newlines between tags - This allows us to use
    'empty' selector in css.
    """

    def _prepare_html(self, text):
        text, type_ = super(ViewPageTemplateFile, self)._prepare_html(text)
        return re.sub(">\s*<", "><", text), type_


class SimplelayoutRenderer(object):

    layout_template = CleanedViewPageTemplateFile('templates/layout.pt')
    slot_template = CleanedViewPageTemplateFile('templates/slot.pt')
    fallbackview = ViewPageTemplateFile('templates/render_block_error.pt')

    def __init__(self, context, storage, name, view=None):

        self.context = context
        self.request = self.context.REQUEST
        self.storage = storage
        self.name = name
        self.view = view

    def render_slot(self):
        return self.slot_template()

    def render_layout(self, index=None, is_update=False):

        self.rows = self.storage.get(self.name, self._one_layout_one_column())

        # If index is None, render all layouts.
        # If not render only a specific layout
        if index is not None:
            if index > len(self.rows) - 1:
                raise ValueError('Layout with index {} does not exists'.format(
                    index))
            else:
                self.rows = [self.rows[index]]

        blocks = self._blocks()
        user_can_edit = self._user_can_edit()

        for row in self.rows:
            config = row.get('config', PersistentMapping())
            if not isinstance(config, PersistentMapping):
                config = config
            else:
                config = config.data

            row['class'] = ' '.join(config.values())
            row['config_json'] = json.dumps(config)

            for col in row['cols']:
                col['class'] = 'sl-column sl-col-{}'.format(len(row['cols']))
                col['blocks'] = filter(
                    lambda block: block.get('uid', '') in blocks,
                    col['blocks'])

                for block in col['blocks']:
                    obj = blocks[block['uid']]
                    self._create_or_update_block(obj, block)

                # Remove hidden blocks for users not having the permission
                # to edit content.
                col['blocks'] = [
                    block for block in col['blocks']
                    if not block['is_hidden'] or block['is_hidden'] and user_can_edit
                ]

        if self.name == 'default':
            for uid, obj in self._blocks_without_state():
                block = self._create_or_update_block(obj, uid=uid)

                # Skip hidden blocks for users not having the permission
                # to edit content.
                if block['is_hidden'] and not user_can_edit:
                    continue

                self.rows[-1]['cols'][-1]['blocks'].append(block)

        return self.layout_template(is_update=is_update)

    def get_simplelayout_settings(self):
        return json.dumps(self._get_sl_settings())

    def get_css_class(self):
        css_classes = ['sl-simplelayout']
        if self._user_can_edit():
            css_classes.append("sl-can-edit")
        return " ".join(css_classes)

    def _user_can_edit(self):
        return api.user.has_permission(
            'Modify portal content',
            obj=self.context) and int(self.request.get('disable_border', 0)) == 0

    def _one_layout_one_column(self):
        return [{"cols": [{"blocks": []}]}]

    def _blocks(self):
        """ Return block objects by UID.
        """

        context = aq_inner(self.context)
        catalog = getToolByName(context, 'portal_catalog')
        blocks = catalog(
            object_provides="ftw.simplelayout.interfaces.ISimplelayoutBlock",
            path={'query': '/'.join(context.getPhysicalPath()), 'depth': 1},
        )
        return {block.UID: block.getObject() for block in blocks}

    def _blocks_without_state(self):
        page_conf = IPageConfiguration(self.context)
        saved_blocks = []

        for container in page_conf.load().values():
            for row in container:
                for col in row['cols']:
                    for block in col['blocks']:
                        if 'uid' in block:
                            saved_blocks.append(block['uid'])

        return filter(lambda x: x[0] not in saved_blocks,
                      self._blocks().items())

    def _create_or_update_block(self, obj, block_dict=None, uid=None):
        if not block_dict:
            block_dict = {}

        if uid:
            block_dict['uid'] = uid

        block_type = normalize_portal_type(obj.portal_type)

        css_classes = ['sl-block', block_type]

        block_is_hidden = False
        hidden_block_support = IHiddenBlock(obj, None)
        if hidden_block_support:
            block_is_hidden = hidden_block_support.is_hidden
            if block_is_hidden:
                css_classes.append('hidden')

        css_classes.extend(getattr(aq_base(obj), 'additional_css_classes', []))

        block_dict['is_hidden'] = block_is_hidden
        block_dict['obj_html'] = self._render_block_html(obj)
        block_dict['type'] = block_type
        block_dict['url'] = obj.absolute_url()
        block_dict['id'] = obj.getId()
        block_dict['css_classes'] = ' '.join(css_classes)

        return block_dict

    def _get_sl_settings(self):
        # 1. global settings
        registry = getUtility(IRegistry)
        settings = json.loads(
            registry.forInterface(ISimplelayoutDefaultSettings,
                                  check=False).slconfig)

        # 2. Update with Permission check
        self._update_permission_related_settings(settings)

        # 3. Update with ISimplelayoutContainerConfig adapter
        adapter = queryMultiAdapter((self.context, self.request),
                                    ISimplelayoutContainerConfig)
        if adapter is not None:
            adapter(settings)

        # 4. View level customizations
        method = 'update_simplelayout_settings'
        if method in dir(self.view) and callable(getattr(self.view, method)):
            getattr(self.view, method)(settings)
        return settings

    def _render_block_html(self, block):

        try:
            html = get_block_html(block)
        except ConflictError:
            raise
        except Exception, exc:
            LOG.exception(exc)
            html = self.fallbackview()

        return html

    def _update_permission_related_settings(self, settings):
        settings['canChangeLayout'] = api.user.has_permission(
            'ftw.simplelayout: Change Layouts',
            obj=self.context)

        # Check if disable_border is in request, if it's there do not load
        # simplelayout.
        settings['canEdit'] = self._user_can_edit()


class BaseSimplelayoutExpression(object):

    def structure(self):
        """ Render a simplelayout slot by name.
        Name is given by the simplelayout expression.
        """
        page_conf = IPageConfiguration(self.context)
        storage = page_conf.load()
        sl_renderer = SimplelayoutRenderer(self.context,
                                           storage,
                                           self.name,
                                           view=self.view)

        return sl_renderer.render_slot()


class SimplelayoutExpression(BaseSimplelayoutExpression,
                             expressions.StringExpr):
    """Simplelayout tales expression
    Usage: <div tal:content='structure simplelayout: default' /> """

    implements(ITALESProviderExpression)

    def __call__(self, econtext):
        self.name = super(SimplelayoutExpression, self).__call__(econtext)
        self.context = econtext.vars['context']
        self.request = econtext.vars['request']
        self.view = econtext.vars['view']

        return self.structure()


try:
    from chameleon.astutil import Symbol
    from chameleon.tales import StringExpr
    from z3c.pt.expressions import ContextExpressionMixin
except ImportError:
    pass
else:
    def render_simplelayout_expression(econtext, name):
        renderer = BaseSimplelayoutExpression()
        renderer.name = name.strip()
        renderer.context = econtext.get('context')
        renderer.request = econtext.get('request')
        renderer.view = econtext.get('view')
        return renderer.structure()

    class ChameleonSimplelayoutExpression(ContextExpressionMixin, StringExpr):
        transform = Symbol(render_simplelayout_expression)



from collections import OrderedDict
from ftw.simplelayout import _
from ftw.simplelayout.browser.ajax.utils import json_response
from ftw.simplelayout.interfaces import ISimplelayoutActions
from ftw.simplelayout.utils import get_block_types
from ftw.simplelayout.utils import normalize_portal_type
from plone.app.content.browser.folderfactories import _allowedTypes
from Products.CMFCore.Expression import Expression
from Products.CMFCore.Expression import getExprContext
from Products.CMFPlone.interfaces.constrains import IConstrainTypes
from zope.component import getMultiAdapter
from zope.component import queryMultiAdapter
from zope.i18n import translate
from zope.publisher.browser import BrowserView


class SimplelayoutToolbox(BrowserView):

    def __call__(self):
        toolbox = {
            'blocks': list(self.addable_blocks()),
            'layoutActions': self.layouts_actions(),
            'labels': self.client_labels()}

        return json_response(self.request, toolbox)

    def addable_blocks(self):
        block_types = get_block_types()
        allowed_types = self._addable_types()

        default_actions = getMultiAdapter((self.context, self.request),
                                          ISimplelayoutActions)

        for fti in block_types:
            if fti in allowed_types:
                add_url = Expression(fti.add_view_expr)(
                    getExprContext(self.context, self.context))
                add_url = add_url.replace('++add++', '++add_block++')

                normalized_portal_type = normalize_portal_type(fti.id)

                specific_actions = queryMultiAdapter(
                    (self.context, self.request),
                    ISimplelayoutActions,
                    name='{0}-actions'.format(normalized_portal_type))

                if specific_actions:
                    actions = specific_actions
                else:
                    actions = default_actions
                yield {
                    'title': translate(msgid=fti.Title(),
                                       domain=fti.i18n_domain,
                                       context=self.request),
                    'description': translate(msgid=fti.Description(),
                                             domain=fti.i18n_domain,
                                             context=self.request),
                    'contentType': normalized_portal_type,
                    'formUrl': add_url,
                    'actions': actions.actions,
                }

    def _addable_types(self):
        allowed_types = _allowedTypes(self.request, self.context)
        constrain = IConstrainTypes(self.context, None)
        if constrain is None:
            return allowed_types
        else:
            locally_allowed = constrain.getLocallyAllowedTypes()
            return [fti for fti in allowed_types
                    if fti.getId() in locally_allowed]

    def layouts_actions(self):
        RELOAD_LAYOUT_URL = self.context.absolute_url() + '/sl-ajax-reload-layout-view'
        return OrderedDict([
            ('move', {
                'title': translate(_(u'label_move_layout',
                                     default=u'Move layout'),
                                   context=self.request),
                'class': 'sl-icon-move move',
            }),
            ('delete', {
                'title': translate(_(u'label_delete_layout',
                                     default=u'Delete layout'),
                                   context=self.request),
                'class': 'sl-icon-delete delete',
            }),
            ('layout-reverse', {
                'title': translate(_(u'label_layout_inverse',
                                     default=u'Invert layout'),
                                   context=self.request),
                'href': RELOAD_LAYOUT_URL,
                'class': 'sl-icon-layout-reverse reload',
                'data-layoutreverse': 'layout-reverse',
                'rules': [2, 3],
            }),
            ('golden-ratio', {
                'title': translate(_(u'label_golden_ratio',
                                     default=u'golden ratio'),
                                   context=self.request),
                'href': RELOAD_LAYOUT_URL,
                'class': 'sl-icon-golden-ratio reload',
                'data-golden_ratio': 'golden-ratio',
                'rules': [2],
            }),
            ('layout13', {
                'title': translate(_(u'label_layout13',
                                     default=u'1-3 layout'),
                                   context=self.request),
                'href': RELOAD_LAYOUT_URL,
                'class': 'sl-icon-layout13 reload',
                'data-layout13': 'layout13',
                'rules': [2],
            }),
            ('layout121', {
                'title': translate(_(u'label_layout121',
                                     default=u'1-2-1 layout'),
                                   context=self.request),
                'href': RELOAD_LAYOUT_URL,
                'class': 'sl-icon-layout121 reload',
                'data-layout121': 'layout121',
                'rules': [3],
            }),
            ('layout112', {
                'title': translate(_(u'label_layout112',
                                     default=u'1-1-2 layout'),
                                   context=self.request),
                'href': RELOAD_LAYOUT_URL,
                'class': 'sl-icon-layout112 reload',
                'data-layout112': 'layout112',
                'rules': [3],
            })
        ])

    def client_labels(self):
        return {
            'labelColumnPostfix': translate(_(u'label_column_postfix',
                                              default=u' Column(s)'),
                                            context=self.request
                                            ),
            'helpDragHint': translate(
                _(u'help_drag_hint',
                  default=u'Please drag, no click required'),
                context=self.request
            )
        }



from ftw.referencewidget.widget import ReferenceBrowserWidget
from ftw.simplelayout import _
from plone.autoform.interfaces import IFormFieldProvider
from plone.directives.form import widget
from plone.supermodel import model
from z3c.relationfield.schema import Relation
from zope import schema
from zope.interface import alsoProvides
from zope.interface import Invalid
from zope.interface import invariant
from zope.interface import provider


class ITeaser(model.Schema):
    """Add internal and external link field."""

    model.fieldset(
        'teaser',
        label=_(u'Teaser'),
        fields=('external_link', 'internal_link'),
        description=_(
            u"description_teaser_fieldset",
            default=(
                u"Use the teaser to redirect to another content or website."
                u"The title and the image of the block will be used to show a link")
        )
    )

    external_link = schema.URI(
        title=_(u'label_external_link', default=u'External URL'),
        required=False)

    widget('internal_link',
           ReferenceBrowserWidget,
           allow_nonsearched_types=True)
    internal_link = Relation(
        title=_(u'label_internal_link', default=u'Internal link'),
        required=False,
    )

    @invariant
    def link_invariant(data):
        if data.external_link and data.internal_link:
            raise Invalid(_(
                u"It's not possible to have an internal_link and an "
                u"external_link together"))

alsoProvides(ITeaser, IFormFieldProvider)


@provider(IFormFieldProvider)
class IHiddenBlock(model.Schema):

    is_hidden = schema.Bool(
        title=_(u'label_is_hidden', default=u'Hide the block'),
        description=_(
            u'description_is_hidden',
            default=u'This will visually hide the block. This is not a '
                    u'security feature, the block and its content can '
                    u'still be accessed.'),
        default=False,
        required=False,
    )



from ftw.builder import Builder
from ftw.builder import create
from ftw.simplelayout.testing import FTW_SIMPLELAYOUT_CONTENT_TESTING
from ftw.simplelayout.testing import FTW_SIMPLELAYOUT_FUNCTIONAL_TESTING
from ftw.simplelayout.testing import SimplelayoutTestCase
from ftw.testbrowser import browsing
from plone.uuid.interfaces import IUUID
import json
import transaction


class TestBlockDelete(SimplelayoutTestCase):

    layer = FTW_SIMPLELAYOUT_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.setup_sample_ftis(self.portal)
        transaction.commit()

        self.contentpage = create(Builder('sample container'))
        # TODO test locking and link integrity (messages).

    def get_payload(self, block):
        block = IUUID(block)
        return {'data': json.dumps({'block': block})}

    @browsing
    def test_delete_block(self, browser):
        block = create(Builder('sample block').within(self.contentpage))
        browser.login().visit(self.contentpage,
                              view='@@sl-ajax-delete-blocks-view',
                              data=self.get_payload(block))

        browser.parse(browser.json['content'])

        browser.find('Delete').click()

        self.assertEquals(0, len(self.contentpage.objectValues()))


class TestRemoveBlockWithinNonSimplelayoutContainer(SimplelayoutTestCase):

    layer = FTW_SIMPLELAYOUT_CONTENT_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.setup_sample_ftis(self.portal)
        self.setup_block_views()

    @browsing
    def test_block_deletion_within_non_simplelayout_container(self, browser):
        """
        This tests ensures that simplelayout blocks can be deleted if they
        are container within a non-simplelayout container.
        """
        # Create a non-simplelayoutish container (this is the case with
        # a `ftw.book.Book` where we first spotted the issue).
        container = create(Builder('folder'))

        # Create a simplelayout block within a non-simplelayoutish
        # container (this is the case with `ftw.book.Chapter` created
        # within a `ftw.book Book` where we first spotted the issue).
        block = create(Builder('sample block').titled('the block').within(container))
        self.assertEqual(['the-block'], container.objectIds())

        # Now delete the block. This triggers
        # `ftw.simplelayout.handlers.update_page_state_on_block_remove` which
        # used to fail if the container was non-simplelayoutish.
        browser.login().visit(container, view='folder_contents')
        browser.visit(block, view='delete_confirmation')
        browser.find_button_by_label('Delete').click()

        # Make sure the block has gone. This is not the real goal of this
        # test. But if we get here, the deletion of the block was successful.
        self.assertEqual([], container.objectIds())



from ftw.simplelayout.images.configuration import Configuration
from ftw.simplelayout.images.cropping.behaviors import IImageCropping
from ftw.simplelayout.images.interfaces import IImageLimits
from ftw.simplelayout.images.interfaces import IImageLimitValidatorMessages
from zope.interface import implementer


class Limits(object):
    limit_configuration = {}

    def __init__(self):
        self._load_limit_configuration()

    def validate(self, limit_type, identifier,
                 width=None, height=None):
        """Validates if the given limit_type (hard/soft) is satisfied for the
        given identifier.

        This method returns true if:

        - there is no configuration for this identifier
        - there is no configuration for the limit_type
        - there is no configuration for the requested dimension
        - the configured limits are respected
        """
        limits = self.get_limits_for(limit_type, identifier)
        if self._validate(width, limits.get('width')) and \
                self._validate(height, limits.get('height')):
            return True
        return False

    def get_limits_for(self, limit_type, identifier):
        """Returns the defined limits for the given identifier:

        i.e. {'width': 100, 'height': 150}
        """
        limit_config = self.limit_configuration.get(identifier, {})
        limits = {
            'width': 0,
            'height': 0
        }
        limits.update(limit_config.get(limit_type, {}))
        return limits

    def get_all_limits_for(self, identifier):
        """Returns all defined limit-types for the given identifier:

        i.e. {
            'hard': {'width': 100, 'height': 150},
            'soft': {'width': 200, 'height': 2000}
        }
        """
        return {
            'hard': self.get_limits_for('hard', identifier),
            'soft': self.get_limits_for('soft', identifier)
        }

    def _validate(self, current, expected):
        if not current or not expected:
            return True
        return current >= expected

    def _load_limit_configuration(self):
        self.limit_configuration = Configuration().image_limits()


@implementer(IImageLimits)
class ImageLimits(object):
    limitsCls = Limits
    image_field_name = 'image'

    def __init__(self, context):
        self.context = context
        self.limits = self.limitsCls()
        self.identifier = self.context.portal_type

    def validate(self, limit_type):
        return self.limits.validate(
            limit_type, self.identifier,
            self._image._width, self._image._height)

    def get_limits_for(self, limit_type):
        return self.limits.get_limits_for(limit_type, self.identifier)

    def get_all_limits(self):
        return self.limits.get_all_limits_for(self.identifier)

    def has_low_quality_image(self):
        """Returns true or false, depending if the soft limit of the image
        of the given context is satisfied or not.
        """
        if not self._image:
            return False
        return not self.validate('soft')

    def low_quality_image_message(self):
        return IImageLimitValidatorMessages(self.context).limit_str(
            'soft', self.identifier, self._image)

    @property
    def _image(self):
        image = getattr(self.context, self.image_field_name)
        if IImageCropping.providedBy(self.context):
            image = self.context.cropped_image or image
        return image



from ftw.builder import Builder
from ftw.builder import create
from ftw.simplelayout.testing import FTW_SIMPLELAYOUT_FUNCTIONAL_TESTING
from ftw.simplelayout.testing import SimplelayoutTestCase
from ftw.trash.trasher import Trasher
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.textfield.value import RichTextValue


class TestTrashIntegration(SimplelayoutTestCase):
    layer = FTW_SIMPLELAYOUT_FUNCTIONAL_TESTING

    def setUp(self):
        super(TestTrashIntegration, self).setUp()
        self.setup_sample_ftis(self.layer['portal'])

    def test_requires_modify_portal_content_permission_for_restoring_blocks(self):
        """ftw.trash by default requires the "Add portal content" permission on the
        parent of an object in order to restore the object.
        For simplelayout blocks we want a different behavior: since we consider blocks
        to be part of the content of their page, restoring blocks should only work when
        the user has the "Modify portal content" permission.
        We wouldn't want users to be able to restore blocks of published pages, the user
        would need to retract or revise the page first.
        """

        setRoles(self.layer['portal'], TEST_USER_ID, ['Manager'])

        page = create(Builder('sample container'))
        block = create(Builder('sample block').within(page))
        Trasher(block).trash()

        # Make sure that the default behavior does not apply by removing the
        # add portal content permission from the user.
        page.manage_permission('Add portal content', roles=[], acquire=False)

        page.manage_permission('Modify portal content', roles=['Manager'], acquire=False)
        self.assertTrue(Trasher(block).is_restorable())

        page.manage_permission('Modify portal content', roles=[], acquire=False)
        self.assertFalse(Trasher(block).is_restorable())

    def test_dont_index_trashed_blocks(self):
        portal = api.portal.get()
        page = create(Builder('sample container'))
        block = create(Builder('sample block').titled(u'fritz').within(page)
                       .having(text=RichTextValue(u'hans')))

        self.assertEquals(1, len(portal.portal_catalog(SearchableText='hans')))

        Trasher(block).trash()
        self.assertEquals(0, len(portal.portal_catalog(SearchableText='hans')))

        Trasher(block).restore()
        self.assertEquals(1, len(portal.portal_catalog(SearchableText='hans')))



import pkg_resources
from zope.i18nmessageid import MessageFactory

_ = MessageFactory('ftw.simplelayout')


try:
    if pkg_resources.get_distribution('ftw.sliderblock').version < '2.0':
        try:
            pkg_resources.require('collective.quickupload')
        except pkg_resources.DistributionNotFound:
            msg = ("ftw.sliderblock < 2.0 requires collective.quickupload which is"
                   " not installed. Either upgrade ftw.sliderblock >= 2.0 "
                   "(recommended) or install collective.upload.")
            raise ImportError(msg)
except pkg_resources.DistributionNotFound:
    pass


def initialize(context):

    from ftw.simplelayout.browser.provider import SimplelayoutExpression
    from Products.Five.browser.pagetemplatefile import getEngine
    from zope.tales.tales import RegistrationError

    zope_engine = getEngine()

    try:
        zope_engine.registerType('simplelayout', SimplelayoutExpression)
    except RegistrationError:
        # zope.reload support
        pass

    try:
        from five.pt import engine
        from ftw.simplelayout.browser.provider import (
            ChameleonSimplelayoutExpression
        )
    except ImportError:
        pass
    else:
        engine.Program.expression_types[
            'simplelayout'] = ChameleonSimplelayoutExpression



from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from ftw.simplelayout.aliasblock.contents.interfaces import IAliasBlock
from plone.app.layout.viewlets.common import ViewletBase
from plone.registry.interfaces import IRegistry
from zope.component import getUtility


def geo_settings_installed():
    has_settings = True
    try:
        from collective.geo.settings.interfaces import IGeoSettings
        getUtility(IRegistry).forInterface(IGeoSettings)
    except (KeyError, ImportError):
        # Getting the record migh yield a KeyError
        has_settings = False

    return has_settings


class AliasBlockFormViewlet(ViewletBase):
    index = ViewPageTemplateFile('templates/aliasblock_form_viewlet.pt')

    def update(self):
        super(AliasBlockFormViewlet, self).update()

        is_aliasblock = IAliasBlock.providedBy(self.context)
        is_aliasblock_add_form = self.view.__name__ == 'ftw.simplelayout.AliasBlock'
        self.do_render_mapwidget = (is_aliasblock or is_aliasblock_add_form) and geo_settings_installed()



from setuptools import setup, find_packages
import os

version = '2.8.4'
maintainer = 'Mathias Leimgruber'

tests_require = [
    'Products.CMFPlone',
    'Products.GenericSetup',
    'ftw.builder',
    'ftw.file',
    'ftw.testbrowser',
    'ftw.testing',
    'ftw.trash',
    'path.py',
    'plone.app.contenttypes',
    'plone.app.lockingbehavior',
    'plone.app.testing',
    'plone.restapi',
    'plone.testing',
    'transaction',
    'zope.configuration',
    'ftw.simplelayout [trash, mapblock]',
    'ftw.logo',
]

extras_require = {
    'tests': tests_require,
    'test': tests_require,
    'contenttypes': [
        # Deprecated
    ],
    'mapblock': [
        'collective.geo.bundle [dexterity]',
    ],
    'plone4': [
        'plone.app.referenceablebehavior',
        'ftw.uploadutility',
        'collective.quickupload',
    ],
    'trash': [
        'ftw.trash',
    ],
    'restapi': [
        'plone.restapi',
    ],
}


setup(name='ftw.simplelayout',
      version=version,
      description='SimpleLayout provides block based content pages',
      long_description=open('README.rst').read() + '\n' +
          open(os.path.join('docs', 'HISTORY.txt')).read(),

      # Get more strings from
      # http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
          'Framework :: Plone',
          'Framework :: Plone :: 4.3',
          'Framework :: Plone :: 5.1',
          'License :: OSI Approved :: GNU General Public License (GPL)',
          'Programming Language :: Python',
          'Programming Language :: Python :: 2.7',
          'Topic :: Software Development :: Libraries :: Python Modules',
          ],

      keywords='ftw plone simplelayout block contentpage',
      author='4teamwork AG',
      author_email='mailto:info@4teamwork.ch',
      maintainer=maintainer,
      url='https://github.com/4teamwork/ftw.simplelayout',
      license='GPL2',

      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['ftw', ],
      include_package_data=True,
      zip_safe=False,

      install_requires=[
          'AccessControl',
          'Acquisition',
          'Persistence',
          'Plone',
          'Products.ATContentTypes',
          'Products.Archetypes',
          'Products.CMFCore',
          'Zope2',
          'collective.dexteritytextindexer',
          'collective.js.jqueryui',
          'ftw.autofeature',
          'ftw.colorbox >= 1.6.0',
          'ftw.iframefix',
          'ftw.referencewidget >= 2.1.0',
          'ftw.table >= 1.22.0',
          'ftw.theming>=1.7.0',
          'ftw.upgrade',
          'plone.api',
          'plone.app.blob',
          'plone.app.dexterity',
          'plone.app.relationfield',
          'plone.autoform',
          'plone.behavior',
          'plone.dexterity',
          'plone.directives.form',
          'plone.uuid',
          'setuptools',
          'z3c.form>=3.2.9',
          'zExceptions',
          'zope.annotation',
          'zope.component',
          'zope.event',
          'zope.i18nmessageid',
          'zope.interface',
          'zope.lifecycleevent',
          'zope.publisher',
          ],

      tests_require=tests_require,
      extras_require=extras_require,

      entry_points="""
      # -*- Entry points: -*-
      [z3c.autoinclude.plugin]
      target = plone
      """,
      )



from ftw.builder import Builder
from ftw.builder import create
from ftw.simplelayout.testing import FTW_SIMPLELAYOUT_FUNCTIONAL_TESTING
from ftw.simplelayout.testing import SimplelayoutTestCase
from ftw.testbrowser import browsing
from plone.uuid.interfaces import IUUID
import json


class TestEditBlock(SimplelayoutTestCase):

    layer = FTW_SIMPLELAYOUT_FUNCTIONAL_TESTING

    def setUp(self):
        self.setup_sample_ftis(self.layer['portal'])
        self.setup_block_views()

        self.page = create(Builder('sample container'))
        self.block = create(Builder('sample block'))

    def get_payload(self, block):
        block = IUUID(block)
        return {'data': json.dumps({'block': block})}

    @browsing
    def test_redirector_redirects_to_edit_view(self, browser):
        browser.login()

        with browser.expect_http_error(reason='Bad Request'):
            browser.visit(self.page,
                          view='sl-ajax-edit-block-view',
                          data={})

        browser.visit(self.page,
                      view='sl-ajax-edit-block-view',
                      data=self.get_payload(self.block))

        self.assertEquals('{0}/edit.json'.format(self.block.absolute_url()),
                          browser.url)

    @browsing
    def test_edit_a_block_returns_json(self, browser):
        browser.login().visit(self.block, view='edit.json')

        self.assertEquals(json.loads(json.dumps(browser.contents)),
                          browser.contents)

    @browsing
    def test_edit_a_block_returns_content_and_proceed(self, browser):
        browser.login().visit(self.block, view='edit.json')
        response = json.loads(browser.contents)

        self.assertIn('content',
                      response,
                      'Response does not contain content.')

        self.assertIn('proceed',
                      response,
                      'Response does not contain proceed.')

    @browsing
    def test_edit_a_block_content_contains_a_form(self, browser):
        browser.login().visit(self.block, view='edit.json')
        response = browser.json
        browser.parse(response['content'])

        self.assertTrue(browser.css('form'), 'No form found in content.')

    @browsing
    def test_edit_a_block_proceed_is_false(self, browser):
        browser.login().visit(self.block, view='edit.json')
        response = browser.json

        self.assertFalse(response['proceed'], 'Proceed should be false.')

    @browsing
    def test_edit_a_block_form_returns_block(self, browser):
        browser.login().visit(self.block, view='edit.json')
        response = browser.json

        browser.parse(response['content'])
        browser.fill({'Title': u'This is a TextBlock',
                      'Text': u'Some text'})
        browser.find_button_by_label('Save').click()

        self.assertEquals(
            {'content': 'OK', 'proceed': True},
            browser.json)

    @browsing
    def test_submit_add_block_traverser_proceed_returns_true(self, browser):
        browser.login().visit(self.block, view='edit.json')
        response = browser.json

        browser.parse(response['content'])
        browser.fill({'Title': u'This is a TextBlock',
                      'Text': u'Some text'})
        browser.find_button_by_label('Save').click()

        response = browser.json
        self.assertTrue(
            response['proceed'], 'Proceed should be true after submitting the form.')



from ftw.upgrade import UpgradeStep


class RemoveNonBehaviorInterfaceFromFTI(UpgradeStep):
    """Remove non behavior interface from FTI.
    """

    def __call__(self):
        self.install_upgrade_profile()



from collections import OrderedDict
from ftw.simplelayout.interfaces import ISimplelayout
from ftw.simplelayout.opengraph.og_site_root import PloneRootOpenGraph
from plone import api
from zope.component import adapts
from zope.interface import Interface


class SimplelayoutPageOpenGraph(PloneRootOpenGraph):
    adapts(ISimplelayout, Interface, Interface)

    def __call__(self):
        """Returns a dict with all og:key, value"""

        return OrderedDict([
            ('og:title', self.get_title()),
            ('og:type', self.get_type()),
            ('og:url', self.get_url()),
            ('og:image', self.get_image_url()),
            ('og:site_name', self.get_site_name()),
            ('og:description', self.get_description()),
            ('fb:app_id', self.get_fb_app_id()),
            ('fb:admins', self.get_fb_admins()),
        ])

    def get_title(self):
        """OG Title"""
        return self._escape(self.context.title_or_id())

    def get_url(self):
        """OG url"""
        return self.context.absolute_url().decode('utf-8')

    def get_image_url(self):
        """OG image"""
        leadimage_view = self.context.restrictedTraverse('@@leadimage')
        if leadimage_view.has_image:
            scale = leadimage_view.get_scale()
            return scale and scale.url or ''
        else:
            return super(SimplelayoutPageOpenGraph, self).get_image_url()

    def get_site_name(self):
        """OG site name"""
        return self._escape(api.portal.get().Title().decode('utf-8'))

    def get_description(self):
        """OG description"""
        return self._escape(self.context.Description().decode('utf-8'))



from zope.component import getMultiAdapter
from ftw.simplelayout.opengraph.interfaces import IOpenGraphDataProvider
from plone.app.layout.viewlets.common import ViewletBase


class OpenGraph(ViewletBase):

    def index(self):
        return '\n'.join(map(
            lambda og: u'<meta property="{0}" content="{1}" />'.format(*og),
            self.get_og_data().items()))

    def get_og_data(self):
        return getMultiAdapter((self.context, self.request, self.view),
                               IOpenGraphDataProvider, name="opengraph")()



from ftw.upgrade import UpgradeStep


class ConfigureChangeLayoutPermissions(UpgradeStep):
    """Configure change layout permissions.
    """

    def __call__(self):
        self.install_upgrade_profile()



from ftw.simplelayout.interfaces import ISimplelayoutDefaultSettings
from ftw.upgrade import UpgradeStep
from plone import api


class FixMissingValueForImageLimitsAndRatios(UpgradeStep):
    """Fix missing value for image limits and ratios.
    """

    def __call__(self):
        self.install_upgrade_profile()
        self.set_default_value_for('image_limits', {})
        self.set_default_value_for('image_cropping_aspect_ratios', {})

    def set_default_value_for(self, name, value):
        if api.portal.get_registry_record(
                name=name, interface=ISimplelayoutDefaultSettings):
            return

        api.portal.set_registry_record(
            name=name,
            value=value,
            interface=ISimplelayoutDefaultSettings)



from ftw.simplelayout.browser.blocks.base import BaseBlock
from ftw.simplelayout.browser.provider import SimplelayoutRenderer
from ftw.simplelayout.interfaces import IPageConfiguration
from ftw.simplelayout.interfaces import ISimplelayout
from ftw.simplelayout.utils import get_block_html
from plone import api
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile


class AliasBlockView(BaseBlock):

    template = ViewPageTemplateFile('templates/aliasblock.pt')

    def __init__(self, context, request):
        super(AliasBlockView, self).__init__(context, request)
        alias = self.context.alias
        self.referenced_obj = alias and alias.to_object or None

    def has_view_permission(self):
        if not self.referenced_obj or self.context.alias.isBroken():
            return False

        return api.user.has_permission('View', obj=self.referenced_obj)

    def can_modify(self):
        return api.user.has_permission('Modify portal content', obj=self.context)

    def referece_is_page(self):
        return ISimplelayout.providedBy(self.referenced_obj)

    def get_referenced_block_content(self):
        """Returns the rendered simplayout content"""
        if self.referece_is_page():
            return self.get_referenced_page_content()
        else:
            return get_block_html(self.referenced_obj)

    def get_referenced_page_content(self):
        page_conf = IPageConfiguration(self.referenced_obj)
        storage = page_conf.load()
        view = self.referenced_obj.restrictedTraverse('view')
        sl_renderer = SimplelayoutRenderer(self.referenced_obj,
                                           storage,
                                           'default',
                                           view=view)
        return sl_renderer.render_layout()



from collections import OrderedDict
from zope.i18n import translate
from ftw.simplelayout import _
from ftw.simplelayout.interfaces import ISimplelayoutActions
from zope.interface import implements


class DefaultActions(object):

    implements(ISimplelayoutActions)

    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.actions = OrderedDict(self.specific_actions().items() +
                                   self.default_actions().items())

    def default_actions(self):
        """Contains default action - move, edit, delete"""
        return OrderedDict([
            ('move', {
                'class': 'move sl-icon-move',
                'title': translate(
                    _(u'label_move_block', default='Move block'),
                    context=self.request),
            }),
            ('edit', {
                'class': 'edit sl-icon-edit',
                'title': translate(
                    _(u'label_edit_block', default='Edit block'),
                    context=self.request),
                'href': self.context.absolute_url() + '/sl-ajax-edit-block-view'
            }),
            ('delete', {
                'class': 'delete sl-icon-delete',
                'title': translate(
                    _(u'label:delete_block', default='Delete block'),
                    context=self.request),
                'href': self.context.absolute_url() + '/sl-ajax-delete-blocks-view'
            })
        ])

    def specific_actions(self):
        return OrderedDict()



from Acquisition import aq_chain
from binascii import a2b_base64
from ftw.simplelayout.browser.ajax.utils import json_response
from ftw.simplelayout.images.configuration import Configuration
from ftw.simplelayout.images.interfaces import IImageLimits
from ftw.simplelayout.interfaces import ISimplelayoutBlock
from ftw.simplelayout.utils import get_block_html
from plone.namedfile.file import NamedBlobImage as NamedBlobImageFile
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.event import notify
from zope.lifecycleevent import ObjectModifiedEvent
import json


class ImageCroppingView(BrowserView):
    template = ViewPageTemplateFile('templates/cropping.pt')
    hard_limit_template = ViewPageTemplateFile('templates/cropping_hard_limit_message.pt')
    soft_limit_template = ViewPageTemplateFile('templates/cropping_soft_limit_message.pt')

    aspect_ratio_configuration = {}

    def __init__(self, context, request):
        super(ImageCroppingView, self).__init__(context, request)
        self._load_aspect_ratio_configuration()
        self.image_limits = IImageLimits(self.context)

    def __call__(self):
        response = {'content': self.template(),
                    'proceed': False}

        if 'form.buttons.save' in self.request.form:
            self.handleSave()
            response['proceed'] = True
            response['content'] = self.get_block_content()

        self.request.response.setHeader("Cache-Control",
                                        "no-cache, no-store, must-revalidate")
        self.request.response.setHeader("Expires", "Sat, 1 Jan 2000 00:00:00 GMT")

        return json_response(self.request, response)

    def handleSave(self):
        is_cropped = json.loads(self.request.form.get('is_cropped', 'false'))

        if is_cropped:
            self.save_cropped_image()
        else:
            self.remove_cropped_image()

        notify(ObjectModifiedEvent(self.context))

    def save_cropped_image(self):
        image_data = self.request.form.get('cropped_image_data')
        config = self.request.form.get('cropped_config')

        contentType, data = self._extract_image_data(image_data)

        self.context.cropped_image = NamedBlobImageFile(
            data=data,
            contentType=contentType,
            filename="abc" + self.context.image.filename)
        self.context.cropped_config = config

    def remove_cropped_image(self):
        self.context.cropped_config = ''
        self.context.cropped_image = None

    def _extract_image_data(self, image_data):
        contentTypePart, dataPart = image_data.split(';')
        key, contentType = contentTypePart.split(':')
        key, data = dataPart.split(',')

        return contentType, a2b_base64(data)

    def aspect_ratios(self):
        return self.aspect_ratio_configuration.get(self.context.portal_type, [])

    def get_block_content(self):
        # It's possible, that we're cropping an image within a block instead an
        # image of a block. In this case, we're not able to return the sl-block
        # html. To fix this issue, we're looking up the next sl-block.
        #
        # This case happens i.e. while cropping a slider pane within the sliderblock.
        return get_block_html(self._sl_block())

    def _load_aspect_ratio_configuration(self):
        self.aspect_ratio_configuration = Configuration().aspect_ratios()

    def hard_limit_validation_template(self):
        return self.hard_limit_template()

    def soft_limit_validation_template(self):
        return self.soft_limit_template()

    def config(self):
        return json.dumps({
            'limits': self.limits(),
            'cropped_config': self.cropped_config()
        })

    def cropped_config(self):
        return json.loads(self.context.cropped_config or '{}')

    def limits(self):
        return self.image_limits.get_all_limits()

    def cropping_image_src(self):
        return '{}/@@images/image?last_modified={}'.format(
            self.context.absolute_url(),
            self.context.modified().strftime('%s')
            )

    def _sl_block(self):
        for obj in aq_chain(self.context):
            if ISimplelayoutBlock.providedBy(obj):
                return obj

        raise AttributeError('No simplelayout block in aq_chain')



from ftw.builder import Builder
from ftw.builder import create
from ftw.simplelayout.testing import FTW_SIMPLELAYOUT_FUNCTIONAL_TESTING
from ftw.simplelayout.testing import SimplelayoutTestCase
from ftw.testbrowser import browsing
from plone import api
from plone.uuid.interfaces import IUUID
import json


class TestInnerEdit(SimplelayoutTestCase):

    layer = FTW_SIMPLELAYOUT_FUNCTIONAL_TESTING

    def setUp(self):
        self.setup_sample_ftis(self.layer['portal'])
        self.setup_block_views()

        types_tool = api.portal.get_tool('portal_types')
        contentpage_fti = types_tool.get('SampleContainer')
        contentpage_fti.allowed_content_types = ('SampleFolderishBlock', )

        self.page = create(Builder('sample container'))
        self.folderishblock = create(Builder('sample folderish block'))

        self.innercontent = create(Builder('sample container')
                                   .within(self.folderishblock))

    def get_payload(self, content):
        uid = IUUID(content)
        return {'data': json.dumps({'uid': uid})}

        browser.visit(self.page,
                      view='sl-ajax-inner-edit-view',
                      data=self.get_payload(self.innercontent))

        self.assertEquals(
            '{0}/inner_edit.json'.format(self.innercontent.absolute_url()),
            browser.url)

    @browsing
    def test_edit_a_block_returns_json(self, browser):
        browser.login().visit(self.innercontent, view='inner_edit.json')

        self.assertEquals(json.loads(json.dumps(browser.contents)),
                          browser.contents)

    @browsing
    def test_edit_an_item_in_block_returns_content_and_proceed(self, browser):
        browser.login().visit(self.innercontent, view='inner_edit.json')
        response = json.loads(browser.contents)

        self.assertIn('content',
                      response,
                      'Response does not contain content.')

        self.assertIn('proceed',
                      response,
                      'Response does not contain proceed.')

    @browsing
    def test_inner_edit_content_contains_a_form(self, browser):
        browser.login().visit(self.innercontent, view='inner_edit.json')
        response = browser.json
        browser.parse(response['content'])

        self.assertTrue(browser.css('form'), 'No form found in content.')

    @browsing
    def test_inner_edit_proceed_is_false(self, browser):
        browser.login().visit(self.innercontent, view='inner_edit.json')
        response = browser.json

        self.assertFalse(response['proceed'], 'Proceed should be false.')

    @browsing
    def test_inner_edit_returns_the_block_content(self, browser):
        browser.login().visit(self.innercontent, view='inner_edit.json')
        response = browser.json

        browser.parse(response['content'])
        browser.fill({'Title': u'This is a title'})
        browser.find_button_by_label('Save').click()
        self.assertEquals({u'content': u'OK', u'proceed': True}, browser.json)

    @browsing
    def test_submit_inner_block_traverser_proceed_returns_true(self, browser):
        browser.login().visit(self.innercontent, view='inner_edit.json')
        response = browser.json

        browser.parse(response['content'])
        browser.fill({'Title': u'This is a title'})
        browser.find_button_by_label('Save').click()
        self.assertEquals({u'content': u'OK', u'proceed': True}, browser.json)



from Acquisition import aq_inner
from Acquisition import aq_parent
from DateTime import DateTime
from ftw.simplelayout.interfaces import IPageConfiguration
from ftw.simplelayout.interfaces import ISimplelayout
from ftw.simplelayout.utils import is_trashed
from persistent.list import PersistentList
from persistent.mapping import PersistentMapping
from plone.uuid.interfaces import IUUID
from Products.CMFPlone.interfaces.siteroot import IPloneSiteRoot
import json


def unwrap_persistence(conf):
    """Unwrap recursice persistent page state
    """
    def unwrap(data):
        if isinstance(data, (PersistentMapping, dict)):
            data = dict(data)
            for key, value in data.items():
                data[key] = unwrap(value)
        elif isinstance(data, (PersistentList, list, tuple, set)):
            return list(map(unwrap, data))
        else:
            # Usually we got basestrings, or integer here, so do nothing.
            pass
        return data
    return unwrap(conf)


def update_page_state_on_copy_paste_block(block, event):
    """Update the uid of the new created block in the page state.
    block: new block
    event.original: origin of the copy event - usually the simplelayout page"""
    parent = aq_parent(block)
    original_parent = event.original

    if not parent:
        return

    if len(parent.getPhysicalPath()) > 1:
        # If we are in a child page of the copied page get the actual parent
        original_parent = event.original.restrictedTraverse(
            '/'.join(parent.getPhysicalPath()[1:]))

    # Only update page state, if the original object is a Simplelayout page.
    if not ISimplelayout.providedBy(original_parent):
        return

    origin_block_uid = IUUID(original_parent.get(block.id))
    page_config = IPageConfiguration(parent)
    page_state = unwrap_persistence(page_config.load())

    new_block_uid = IUUID(block)
    new_page_state = json.loads(
        json.dumps(page_state).replace(origin_block_uid,
                                       new_block_uid))

    # We should not update object positions here, because:
    # 1. "Ordered folder" makes sure that the order is the same as before
    #    when copy / pasting.
    # 2. Updating positions does not work here, because our objects are not
    #    acquisition wrappable yet (since not yet pasted) and the updating
    #    mechanism will trigger events (such as plone.app.referenceablebehavior),
    #    which require acquisition.
    page_config.store(new_page_state, update_positions=False)


def update_page_state_on_block_remove(block, event):

    if event.newParent is None:
        # Be sure it's not cut/paste
        block_uid = IUUID(block)
        parent = aq_parent(aq_inner(block))

        # Do nothing if the event wasn't fired by the block's parent.
        # This happens when an ancestor is deleted, e.g. the Plone site itself.
        if parent is not event.oldParent:
            return

        config = IPageConfiguration(parent, None)
        if not config:
            # We are not able to update the page state if the parent
            # of the block is not simplelayoutish at all.
            return
        page_state = config.load()

        for container in page_state.values():
            for layout in container:
                for column in layout['cols']:
                    cache_amound_blocks = len(column['blocks'])
                    column['blocks'] = [item for item in column['blocks']
                                        if item['uid'] != block_uid]
                    if cache_amound_blocks != len(column['blocks']):
                        # Block has been removed
                        break
        config.store(page_state)


def modify_parent_on_block_edit(block, event, idxs=None):
    parent = aq_parent(aq_inner(block))

    # Parent may be None.
    # Example: ObjectGeoreferencedEvent triggers Modified event, but the obj is
    # not yet added to a container
    if parent is None:
        return

    if IPloneSiteRoot.providedBy(parent):
        parent.setModificationDate(DateTime())
    else:
        idxs = idxs if idxs else []
        parent.reindexObject(idxs=idxs)

    try:
        from collective.lastmodifier.utils import set_last_modifier
        set_last_modifier(parent)
    except ImportError:
        pass


def handle_trashed_and_restored_blocks(block, event):
    modify_parent_on_block_edit(block, event, idxs=['SearchableText'])



from ftw.simplelayout import _
from ftw.simplelayout.interfaces import ISimplelayoutDefaultSettings
from plone.app.registry.browser.controlpanel import ControlPanelFormWrapper
from plone.app.registry.browser.controlpanel import RegistryEditForm
from plone.z3cform import layout
from z3c.form import form


class SimplelayoutDefaultSettingsForm(RegistryEditForm):
    form.extends(RegistryEditForm)
    schema = ISimplelayoutDefaultSettings

SimplelayoutDefaultSettingsView = layout.wrap_form(
    SimplelayoutDefaultSettingsForm,
    ControlPanelFormWrapper)
SimplelayoutDefaultSettingsView.label = _(u'Simplelayout settings')



from collective.geo.behaviour import MessageFactory as _
from ftw.simplelayout.mapblock.browser.widget import BlockMapFieldWidget
from plone.autoform import directives as form
from plone.autoform.interfaces import IFormFieldProvider
from plone.supermodel import model
from zope import schema
from zope.interface import alsoProvides


class IBlockCoordinates(model.Schema):
    """Add coordinates and map styles to content
    """

    coordinates = schema.Text(
        title=_(u"Coordinates"),
        description=_(u"Modify geographical data for this content"),
        required=False,
    )

    form.widget(
        coordinates=BlockMapFieldWidget
    )

    model.fieldset(
        'coordinates',
        label=_(u'Coordinates'),
        fields=('coordinates', )
    )


alsoProvides(IBlockCoordinates, IFormFieldProvider)



from ftw.simplelayout.browser.ajax.utils import json_response
from ftw.simplelayout.browser.provider import SimplelayoutRenderer
from ftw.simplelayout.interfaces import IPageConfiguration
from persistent.mapping import PersistentMapping
from zExceptions import BadRequest
from zope.publisher.browser import BrowserView
import json


class ReloadLayoutView(BrowserView):
    """Reloads a layout by name and index also performs some modifications"""

    def __init__(self, context, request):
        super(ReloadLayoutView, self).__init__(context, request)
        self.data = None

    def __call__(self):
        self.data = json.loads(self.request.get('data', '{}'))

        if 'name' not in self.data or 'layoutindex' not in self.data:
            raise BadRequest('"name" and "layoutindex" are required')

        self.name = self.data['name']
        self.layoutindex = int(self.data['layoutindex'])
        self.set_layout_state()
        return json_response(self.request,
                             {'content': self.render_new_layout()})

    def set_layout_state(self):
        page_conf = IPageConfiguration(self.context)
        state = page_conf.load()

        if 'config' not in self.data:
            return

        current_config = state.get(self.name)[self.layoutindex].get(
            'config', PersistentMapping())

        for key, value in self.data['config'].items():
            if key in current_config:
                del current_config[key]
            else:
                current_config[key] = value

        state.get(
            self.name)[self.layoutindex]['config'] = current_config
        page_conf.store(state)

    def render_new_layout(self):
        page_conf = IPageConfiguration(self.context)
        sl_renderer = SimplelayoutRenderer(self.context,
                                           page_conf.load(),
                                           self.name)
        return sl_renderer.render_layout(index=self.layoutindex,
                                         is_update=True)



from plone.restapi.services import Service
from ftw.simplelayout.configuration import synchronize_page_config_with_blocks


class SynchronizePageConfigWithBlocks(Service):

    def reply(self):
        return synchronize_page_config_with_blocks(self.context)



from ftw.simplelayout.browser.blocks.base import BaseBlock
from ftw.simplelayout.contenttypes.contents.videoblock import is_vimeo_url
from ftw.simplelayout.contenttypes.contents.videoblock import is_youtube_nocookie_url
from ftw.simplelayout.contenttypes.contents.videoblock import is_youtube_url
from plone.uuid.interfaces import IUUID
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from urlparse import parse_qs
from urlparse import urlparse
import json


VIMEO_PLAYER = "//player.vimeo.com/video/{0}"
YOUTUBE_NO_COOKIE_PLAYER = 'https://www.youtube-nocookie.com/embed/{0}'


class VideoBlockView(BaseBlock):

    youtube_template = ViewPageTemplateFile('templates/videoblock_youtube.pt')
    youtube_nocookie_template = ViewPageTemplateFile('templates/videoblock_youtube_nocookie.pt')
    vimeo_template = ViewPageTemplateFile('templates/videoblock_vimeo.pt')

    template = None

    def __call__(self):
        if is_youtube_url(self.context.video_url):
            self.template = self.youtube_template
        elif is_youtube_nocookie_url(self.context.video_url):
            self.template = self.youtube_nocookie_template
        elif is_vimeo_url(self.context.video_url):
            self.template = self.vimeo_template
        else:
            raise ValueError("No template found.")

        return super(VideoBlockView, self).__call__()

    def get_uuid(self):
        return 'uuid_{0}'.format(IUUID(self.context))

    def youtube_config(self):
        config = {'videoId': self.get_video_id()}

        self.updateStartSeconds(config)

        return json.dumps(config)

    def vimeo_player(self):
        return VIMEO_PLAYER.format(self.get_video_id())

    def youtbube_no_cookie_player(self):
        return YOUTUBE_NO_COOKIE_PLAYER.format(self.get_video_id())

    def get_video_id(self):
        if is_youtube_url(self.context.video_url):
            parsed_url = urlparse(self.context.video_url)
            return parsed_url.path[1:]
        elif is_vimeo_url(self.context.video_url):
            parsed_url = urlparse(self.context.video_url)
            path = parsed_url.path.split('/')
            return path[-1]
        elif is_youtube_nocookie_url(self.context.video_url):
            parsed_url = urlparse(self.context.video_url)
            path = parsed_url.path.split('/')
            return path[-1]
        else:
            return None

    def updateStartSeconds(self, config):
        """Adds the start option if `start` or `t` is available in 
        query string. Only works for youtube player
        """
        if is_youtube_url(self.context.video_url):
            qs = parse_qs(urlparse(self.context.video_url).query)
            if 'start' in qs:
                config['playerVars-start'] = int(qs['start'][0])
            if 't' in qs:
                config['playerVars-start'] = int(qs['t'][0])

    def player_height(self):
        return self.context.player_height or '500px'



from ftw.builder import Builder
from ftw.builder import create
from ftw.simplelayout.testing import FTW_SIMPLELAYOUT_CONTENT_TESTING
from unittest import TestCase
import json


class TestAddableBlocksDefaultContent(TestCase):

    layer = FTW_SIMPLELAYOUT_CONTENT_TESTING

    def setUp(self):
        super(TestAddableBlocksDefaultContent, self).setUp()

    def test_addable_blocks_view(self):
        page = create(Builder('sl content page'))

        view = page.restrictedTraverse('@@sl-toolbox-view')

        result = [item.get('contentType')
                  for item in list(view.addable_blocks())]
        result.sort()

        self.assertEquals(
            ['ftw-simplelayout-aliasblock',
             'ftw-simplelayout-filelistingblock',
             'ftw-simplelayout-galleryblock',
             'ftw-simplelayout-mapblock',
             'ftw-simplelayout-textblock',
             'ftw-simplelayout-videoblock'],
            result)

    def test_addable_blocks_json(self):
        page = create(Builder('sl content page'))
        view = page.restrictedTraverse('@@sl-toolbox-view')

        addable_types_json = json.loads(view())['blocks']

        self.maxDiff = None

        self.assertDictEqual(
            {u'title': u'TextBlock',
             u'description': u'The text block renders text and images.',
             u'contentType': u'ftw-simplelayout-textblock',
             u'actions': {u'cropping': {u'class': u'sl-icon-crop crop-image',
                                        u'href': u'./sl-ajax-crop-image',
                                        u'title': u'Crop image'},
                          u'delete': {u'class': u'delete sl-icon-delete',
                                      u'href': u'http://nohost/plone/ftw-simplelayout-contentpage/sl-ajax-delete-blocks-view',
                                      u'title': u'Delete block'},
                          u'edit': {u'class': u'edit sl-icon-edit',
                                    u'href': u'http://nohost/plone/ftw-simplelayout-contentpage/sl-ajax-edit-block-view',
                                    u'title': u'Edit block'},
                          u'image': {u'class': u'sl-icon-image block-server-action',
                                     u'data-scale': u'sl_textblock_large',
                                     u'data-imagefloat': u'no-float',
                                     u'href': u'./sl-ajax-reload-block-view',
                                     u'title': u'Image without floating'},
                          u'imageLeft': {u'class': u'sl-icon-image-left block-server-action',
                                         u'data-scale': u'sl_textblock_small',
                                         u'data-imagefloat': u'left',
                                         u'href': u'./sl-ajax-reload-block-view',
                                         u'title': u'Float image left'},
                          u'imageLeftLarge': {u'class': u'sl-icon-image-left-large block-server-action',
                                              u'data-scale': u'sl_textblock_middle',
                                              u'data-imagefloat': u'left',
                                              u'href': u'./sl-ajax-reload-block-view',
                                              u'title': u'Float large image left'},
                          u'imageRight': {u'class': u'sl-icon-image-right block-server-action',
                                          u'data-scale': u'sl_textblock_small',
                                          u'data-imagefloat': u'right',
                                          u'href': u'./sl-ajax-reload-block-view',
                                          u'title': u'Float image right'},
                          u'imageRightLarge': {u'class': u'sl-icon-image-right-large block-server-action',
                                               u'data-scale': u'sl_textblock_middle',
                                               u'data-imagefloat': u'right',
                                               u'href': u'./sl-ajax-reload-block-view',
                                               u'title': u'Float large image right'},
                          u'move': {u'class': u'move sl-icon-move',
                                    u'title': u'Move block'}},
             u'formUrl': u'{0}/++add_block++ftw.simplelayout.TextBlock'.format(
                 page.absolute_url())
             },
            addable_types_json[0])

        self.assertDictEqual(
            {u'title': u'FileListingBlock',
             u'description': u'The file listing block renders a list of uploaded files with configurable header which can be used to change the order of the listing.',
             u'contentType': u'ftw-simplelayout-filelistingblock',
             u'actions': {u'delete': {u'class': u'delete sl-icon-delete',
                                      u'href': u'http://nohost/plone/ftw-simplelayout-contentpage/sl-ajax-delete-blocks-view',
                                      u'title': u'Delete block'},
                          u'edit': {u'class': u'edit sl-icon-edit',
                                    u'href': u'http://nohost/plone/ftw-simplelayout-contentpage/sl-ajax-edit-block-view',
                                    u'title': u'Edit block'},
                          u'folderContents': {u'class': u'sl-icon-folder-contents redirect',
                                              u'href': u'/folder_contents',
                                              u'title': u'Go to folder contents for managing files'},
                          u'move': {u'class': u'move sl-icon-move',
                                    u'title': u'Move block'}},
             u'formUrl': u'{0}/++add_block++ftw.simplelayout.FileListingBlock'.format(
                 page.absolute_url())
             },
            addable_types_json[1])



from ftw.builder import Builder
from ftw.builder import create
from ftw.simplelayout.contenttypes.contents.videoblock import is_vimeo_url
from ftw.simplelayout.contenttypes.contents.videoblock import is_youtube_url
from ftw.simplelayout.contenttypes.contents.videoblock import is_youtube_nocookie_url
from ftw.simplelayout.testing import FTW_SIMPLELAYOUT_CONTENT_TESTING
from ftw.testbrowser import browsing
from ftw.testbrowser.pages import factoriesmenu
from ftw.testbrowser.pages import statusmessages
from plone.uuid.interfaces import IUUID
from unittest import TestCase
import json


class TestVideoValidators(TestCase):

    def test_valid_youtube_url(self):
        url = 'https://youtu.be/W42x6-Wf3Cs'
        self.assertTrue(is_youtube_url(url))

        url = 'http://youtu.be/W42x6-Wf3Cs'
        self.assertTrue(is_youtube_url(url))

    def test_invalid_youtube_url(self):
        url = 'https://example.com/W42x6-Wf3Cs'
        self.assertFalse(is_youtube_url(url))

        url = 'https://example.com/W42x6-Wf3Cs/something'
        self.assertFalse(is_youtube_url(url))

        url = 'https://youtu.be/W42x6-Wf3Cs/something'
        self.assertFalse(is_youtube_url(url))

        url = 'https://youtu.be'
        self.assertFalse(is_youtube_url(url))

        url = 'https://youtu.be/'
        self.assertFalse(is_youtube_url(url))

    def test_valid_vimeo_url(self):
        url = 'https://vimeo.com/channels/staffpicks/128510631'
        self.assertTrue(is_vimeo_url(url))

        url = 'http://vimeo.com/channels/staffpicks/128510631'
        self.assertTrue(is_vimeo_url(url))

    def test_invalid_vimeo_url(self):
        url = 'https://vimeo.com/channels/staffpicks/128510631something'
        self.assertFalse(is_vimeo_url(url))

        url = 'https://example.com/channels/staffpicks/128510631'
        self.assertFalse(is_vimeo_url(url))

        url = 'http://vimeo.com/channels/staffpicks/128510631/someting'
        self.assertFalse(is_vimeo_url(url))

        url = 'http://vimeo.com'
        self.assertFalse(is_vimeo_url(url))

        url = 'http://vimeo.com/channels/'
        self.assertFalse(is_vimeo_url(url))

    def test_valid_youtube_nocookie_url(self):
        url = 'https://www.youtube-nocookie.com/embed/UUrddqT9i_s'
        self.assertTrue(is_youtube_nocookie_url(url))


class TestTextBlockRendering(TestCase):

    layer = FTW_SIMPLELAYOUT_CONTENT_TESTING

    def setUp(self):
        self.page = create(Builder('sl content page').titled(u'A page'))

    @browsing
    def test_adding_youtube_videoblock(self, browser):
        browser.login().visit(self.page)
        factoriesmenu.add('VideoBlock')
        browser.fill({'Youtube, or Vimeo URL': 'https://youtu.be/W42x6-Wf3Cs'})
        browser.find_button_by_label('Save').click()
        self.assertTrue(browser.css('.sl-youtube-video'))

    @browsing
    def test_adding_vimeo_videoblock(self, browser):
        browser.login().visit(self.page)
        factoriesmenu.add('VideoBlock')
        browser.fill({'Youtube, or Vimeo URL': 'https://vimeo.com/channels/staffpicks/128510631'})
        browser.find_button_by_label('Save').click()
        self.assertTrue(browser.css('iframe'))

    @browsing
    def test_youtube_video_view_has_id_based_on_uuid(self, browser):
        videoblock = create(Builder('sl videoblock')
                            .having(video_url='https://youtu.be/W42x6-Wf3Cs')
                            .within(self.page))

        browser.login().visit(videoblock, view='@@block_view')
        self.assertTrue(browser.css('#uuid_{0}'.format(IUUID(videoblock))))

    @browsing
    def test_youtube_video_view_has_videoid(self, browser):
        videoblock = create(Builder('sl videoblock')
                            .having(video_url='https://youtu.be/W42x6-Wf3Cs')
                            .within(self.page))

        browser.login().visit(videoblock, view='@@block_view')
        youtube_config = json.loads(browser.css(
            '.sl-youtube-video').first.attrib['data-youtube'])

        self.assertEquals('W42x6-Wf3Cs', youtube_config['videoId'])

    @browsing
    def test_youtube_video_view_has_startSeconds_based_on_querystring(self, browser):
        videoblock = create(Builder('sl videoblock')
                            .having(video_url='https://youtu.be/W42x6-Wf3Cs?start=5')
                            .within(self.page))

        browser.login().visit(videoblock, view='@@block_view')
        youtube_config = json.loads(browser.css(
            '.sl-youtube-video').first.attrib['data-youtube'])

        self.assertEquals(5, youtube_config['playerVars-start'])

        videoblock2 = create(Builder('sl videoblock')
                             .having(video_url='https://youtu.be/W42x6-Wf3Cs?t=5')
                             .within(self.page))

        browser.login().visit(videoblock2, view='@@block_view')
        youtube_config = json.loads(browser.css(
            '.sl-youtube-video').first.attrib['data-youtube'])

        self.assertEquals(5, youtube_config['playerVars-start'])

    @browsing
    def test_vimeo_view_iframe_src_contains_videoid(self, browser):
        videoblock = create(Builder('sl videoblock')
                            .having(video_url='https://vimeo.com/channels/staffpicks/128510631')
                            .within(self.page))

        browser.login().visit(videoblock, view='@@block_view')
        iframe_src = browser.css('iframe').first.attrib['src'].split('/')
        self.assertEquals('128510631', iframe_src[-1])

    @browsing
    def test_video_url_invariant(self, browser):
        browser.login().visit(self.page)
        factoriesmenu.add('VideoBlock')
        browser.fill({'Youtube, or Vimeo URL': 'https://example.com'})
        browser.find_button_by_label('Save').click()

        statusmessages.assert_message('There were some errors.')
        self.assertEquals('This is no a valid youtube, or vimeo url.',
                          browser.css('.field.error').first.text)

    @browsing
    def test_raise_valueerror_if_template_is_undeterminable(self, browser):
        videoblock = create(Builder('sl videoblock')
                            .having(video_url='https://example.com')
                            .within(self.page))

        browser.exception_bubbling = True
        with self.assertRaises(ValueError):
            browser.login().visit(videoblock, view='@@block_view')

        videoblock_view = videoblock.restrictedTraverse('@@block_view')
        self.assertIsNone(videoblock_view.get_video_id())

    @browsing
    def test_youtube_video_block_title(self, browser):
        videoblock = create(Builder('sl videoblock')
                            .having(video_url='https://youtu.be/W42x6-Wf3Cs')
                            .titled(u'A video on Youtube')
                            .within(self.page))

        browser.login()

        # Make sure the title is not rendered by default.
        browser.visit(videoblock, view='@@block_view')
        self.assertEqual([], browser.css('h2'))

        # Configure the block to render the title.
        browser.visit(videoblock, view='edit')
        browser.fill({'Show title': True}).submit()
        browser.visit(videoblock, view='@@block_view')
        self.assertEquals(
            'A video on Youtube',
            browser.css('h2').first.text
        )

    @browsing
    def test_vimeo_block_title(self, browser):
        videoblock = create(Builder('sl videoblock')
                            .having(video_url='https://vimeo.com/channels/staffpicks/128510631')
                            .titled(u'A video on Vimeo')
                            .within(self.page))

        browser.login()

        # Make sure the title is not rendered by default.
        browser.visit(videoblock, view='@@block_view')
        self.assertEqual([], browser.css('h2'))

        # Configure the block to render the title.
        browser.visit(videoblock, view='edit')
        browser.fill({'Show title': True}).submit()
        browser.visit(videoblock, view='@@block_view')
        self.assertEquals(
            'A video on Vimeo',
            browser.css('h2').first.text
        )



from ftw.builder import Builder
from ftw.builder import create
from ftw.simplelayout.interfaces import IPageConfiguration
from ftw.simplelayout.testing import FTW_SIMPLELAYOUT_FUNCTIONAL_TESTING
from ftw.simplelayout.testing import SimplelayoutTestCase
from ftw.testbrowser import browsing
from plone.uuid.interfaces import IUUID
import transaction


class TestRemoveBlockPloneUI(SimplelayoutTestCase):

    layer = FTW_SIMPLELAYOUT_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.setup_sample_ftis(self.portal)
        self.setup_block_views()

        self.page = create(Builder('sample container'))
        self.block = create(Builder('sample block').within(self.page))

        self.page_state = {
            "default": [
                {"cols": [
                    {"blocks": [
                        {"uid": IUUID(self.block)}
                    ]}
                ]}
            ]
        }

        IPageConfiguration(self.page).store(self.page_state)
        transaction.commit()

    @browsing
    def test_page_state_is_updated_after_block_removal(self, browser):
        browser.login().visit(self.page, view='folder_contents')

        browser.visit(self.block, view='delete_confirmation')
        browser.find_button_by_label('Delete').click()
        browser.visit(self.page)
        self.assertFalse(browser.css('.sl-block'))
